
#---------------------
# Read covariates
# Sanqian Zhang and Leontine Alkema, 2015
#---------------------

ReadCovariates <- function(
  # infofile containing country codes
  infofile = "data/input/country_info_20150824.csv",
  # lifetable  
  LTfile = "data/input/LTmmr_20151023.dta",
  # GDP 
  gdpfile = "data/input/gdp.csv",
  # 2015 SAB database
  sabfile = "data/input/SAB_2015database.csv", 
  # WPP2015 estimates
  wpp2015est = "data/input/WPP2015_data.csv"
  ){

  year.t <- seq(1985, 2015)
  nyears <- length(year.t)
  #---------------------
  # get countries of interest from country sheet
  metadat <- read.csv(infofile)
  select <- metadat$in.study.15
  
  iso.c <- metadat$iso.code[select]
  numcode.c <- metadat$num.code[select]
  C <- length(iso.c)
  name.c <- metadat$country.name[select]
  group.c <- metadat$group.12[select] # MMEIG2014 grouping (no longer used)
  
  # Regions used for the modeling
  reg.c <- metadat$regions.for.regr[select]
  regnames <- unique(reg.c)
  R <- length(regnames)
  getr.c <- Getc.i(iso.i = paste(reg.c), iso.c = regnames)
  
  mdg.c <- metadat$mdg.region.small[select]
  whocode.c <- metadat$whocode[select]
  
  #--------------------------
  print("Reading in lifetable information...")
  # read WHO life tables
  datlf <- read.dta(file = LTfile)
  
  # x refers to age
  datlf2 <- datlf[is.element(datlf$x, seq(15,45,5)),]
  remove(datlf)
  
  a.ct <- deaths.ct <- l15.ct<- t15t50.ct <- matrix(NA, C, nyears)
  for (c in 1:C){
    for (t in 1:nyears){
      select <- datlf2$year==year.t[t] & paste(datlf2$iso3) == paste(iso.c[c])
      deaths.ct[c,t] <- sum(datlf2$dth[select])
      a.ct[c,t] <- sum(datlf2$hivdths[select])/sum(datlf2$dth[select])
      l15.ct[c,t] <- datlf2[select & datlf2$x==15,"l"]
      t15t50.ct[c,t] <- sum(datlf2$L[select])
    }}

  
  #--------------------------
  # read_covariates and births
  print("Reading in covariates...")
  datc <- read.csv(gdpfile)
  wpp2015est <- read.csv(wpp2015est)
  sabnew <- read.csv(sabfile)
  name.h <- c("logGDP", "logGFR", "SAB")
  H <- length(name.h)
  births.ct <-  gfr.ct <- gfr2015.ct <- logGDP.ct <- sab.ct <- births2015.ct <- matrix(NA, C, nyears)
  
  for (c in 1:C){
    for (t in 1:nyears){
      select <- which(paste(datc$isocode)==iso.c[c] & datc$middate == (year.t[t] +0.5))
      logGDP.ct[c,t] <- log(datc[select, c("gdp_who_5yrma")]) 
      
      select2 <- which(wpp2015est$CountryCode==numcode.c[c] & wpp2015est$Time==year.t[t])[1]
      births2015.ct[c,t] <- wpp2015est[select2,"Births"]
      gfr2015.ct[c,t] <- wpp2015est[select2,"GFR"]
      
      select3 <- which (sabnew$iso.code== paste(iso.c[c]) & sabnew$mid.date == (year.t[t] +0.5))
      sab.ct[c,t] <- sabnew$sab.est[select3]
    }
  }
  
  
  
  #----------------------------
  meta <- list(iso.c = iso.c, year.t= year.t, a.ct = a.ct,  
               births.ct = births2015.ct, gfr2015.ct = gfr2015.ct,
               numcode.c = numcode.c, 
               l15.ct = l15.ct, t15t50.ct = t15t50.ct,
               logGDP.ct=logGDP.ct,  sab.ct = sab.ct, 
               name.h = name.h,
               deaths.ct = deaths.ct, name.c = name.c, mdg.c = mdg.c, C = C,nyears = nyears, 
               group.c = group.c, regnames = regnames, reg.c = reg.c, R = R, getr.c = getr.c, whocode.c = whocode.c, year.t = year.t
               ##HIV related parameters
               ,caids = 1, uaids = 0.3, kaids = 0.3
  )
  
  # information related to HIV 
  v2015.ct<-matrix(NA,nrow=meta$C,ncol=meta$nyears)
  for (c in 1:meta$C){
    for (t in 1:meta$nyears){
      v2015.ct[c,t] <- meta$caids*meta$kaids*meta$gfr2015.ct[c,t]/(1+meta$caids*(meta$kaids-1)*meta$gfr2015.ct[c,t])
    }}
  
  meta$v2015.ct <- v2015.ct
  
  # matrix of covariates
  H <- length(meta$name.h)
  Xunst.cth <- X.cth <- array(NA, c(meta$C, meta$nyears, H))
  
  dimnames(Xunst.cth)[[3]] <- meta$name.h
  
  Xunst.cth[,,"logGDP"] <- meta$logGDP.ct
  Xunst.cth[,,"logGFR"] <- log(meta$gfr2015.ct)
  Xunst.cth[,,"SAB"] <- meta$sab.ct
  # standardize X to be centered at zero
  meta$means.h <-   apply(Xunst.cth, 3, mean) 
  for (h in 1:H){
    X.cth[,,h] <- Xunst.cth[,,h] - meta$means.h[h]  
  }
  dimnames(X.cth)[[3]] <- meta$name.h
  meta$X.cth <- X.cth
  
  save(meta, file = paste0("data/interm/meta.rda"))
  print("Object meta written to data/interm/meta.rda")
  return(NULL)
  
}


#----------------------------
# The End!

