
#---------------------
# Read MMR data
# Sanqian Zhang and Leontine Alkema, 2015
#---------------------

ReadMMRdata <- function(
  # data files with observed maternal mortality records
  vrfilename = "data/input/observedmmr/VRinput1.csv",
  vraddfile = "data/input/observedmmr/VRinput2.csv",
  CIfile = "data/input/observedmmr/SpecStudyinput.csv",
  dhsfile = "data/input/observedmmr/DHSinput.csv",
  miscfile = "data/input/observedmmr/Miscinput.csv",
  
  vr2014 = "data/input/observedmmr/VRinput2014.csv",
  # cutoff for usability criteria for VR inclusion
  vrusa.cutoff = 0.6,
  # cutoff for usability criteria for window rule in VR inclusion
  vrusa.neighbourhood.cutoff = 0.6,
  
  #location of meta 
  metaobject = "data/interm/meta.rda"
){
  
load(metaobject)  
data.L <- list()

#--------------------------------
# read VR data
datvrall<-  read.csv(vrfilename,header=TRUE)
datvr <- datvrall[is.element(datvrall$Country, meta$whocode.c), ]
# set inclusion to false for entries without age breakdown and years before 1985
include.j <- datvr$Frmat != 9 & datvr$Year>=1985

# find isos by merging with who codes
# use temp j subscript
getc.j <- Getc.i(iso.i = datvr$Country, iso.c = paste(meta$whocode.c))
iso.j <- meta$iso.c[getc.j]
year.j <- datvr$Year
isicd10.j <- is.element(datvr$List, c("103", "104", "10M","101"))
cause.j <- datvr$Cause

# ages: sum up deaths between ages 15 and 49
# note that for format==7 and 8, the age group is 45-54
# 9 is no age breakdown

columnstosum <- which(is.element(names(datvr), paste("A", seq(15,45,5), sep = "")))
totals.j <- apply(datvr[, columnstosum],1, sum)

# deal with deaths with unknown ages
unknown.j <- datvr$unk 
unknown.j[is.na(unknown.j)] <- 0
# for maternal deaths, add those in unknown ages to the total maternal deaths
# for total deaths, redistribute according to age pattern of deaths
totals.j <- ifelse(is.element(cause.j, c("O96", "O97", "mat")), totals.j + unknown.j, totals.j + unknown.j*totals.j/datvr$tot)
names(totals.j) <- cause.j 

late.k <- iso.k <- year.k <- tot.k <- matincllate.k <- ill.k <- include.k <- NULL 
isos <- NULL
for (iso in unique(iso.j)){
  for (year in unique(year.j[iso.j==iso])){
    select <- iso.j==iso & year.j==year
    iso.k <- c(iso.k, iso)  
    year.k <- c(year.k, year)
    if (sum(select & cause.j == "all")!=1) {
      # exclude country-year if there are multiple VR entries in all deaths for the same country in the same year
      tot.k <- c(tot.k, totals.j[select & cause.j == "all"][1])
      include.k <- c(include.k, FALSE) 
      ill.k <- c(ill.k, ifelse(sum(select & cause.j == "ill")==1, totals.j[select & cause.j == "ill"],0))
      matincllate.k <- c(matincllate.k, ifelse(sum(select & cause.j == "mat") ==1, totals.j[select & cause.j == "mat"],0))
    } else {
      tot.k <- c(tot.k, totals.j[select & cause.j == "all"])
      include.k <- c(include.k, include.j[select& cause.j == "all"])
      ill.k <- c(ill.k, ifelse(sum(select & cause.j == "ill") ==1, totals.j[select & cause.j == "ill"],0))
      matincllate.k <- c(matincllate.k, ifelse(sum(select & cause.j == "mat") ==1, totals.j[select & cause.j == "mat"],0))
      if (!isicd10.j[select][1]){
        late.k <- c(late.k,NA)
      } else {
        # 096 and O97 are late maternal
        late.k <- c(late.k,sum(totals.j[select & is.element(cause.j , c("O96", "O97"))])) 
      }
    }
  }
}

ill.k[is.na(ill.k)] <- 0
K <- length(iso.k)
propill.k <- ill.k/tot.k

getc.k <- Getc.i(iso.i = iso.k, iso.c = meta$iso.c)
gett.k <- Gett.i(years.i = year.k, year.t = meta$year.t)

complete.k <- rep(NA, K)
for (k in 1:K){
  complete.k[k] <- min(1,tot.k[k]/meta$deaths.ct[getc.k[k], gett.k[k]])
}
usa.k <- complete.k*(1-propill.k)
mat.touse.k = ifelse(is.na(late.k),matincllate.k,matincllate.k-late.k)
excl.k <- as.character(ifelse(include.k==1,NA,"noage"))
# note reason for exclusion 
excl.k[which(iso.k=="CHN")]<- "mmeigcheck"  #for china data, exclude due to MMEIG data assessment 

data.L[["vr"]] <- data.frame(iso = iso.k, year = year.k+0.5,start = year.k, end = year.k+1,
                             data.start=year.k,data.end=year.k+1,
                             mat = mat.touse.k,
                             mat.incl.late = matincllate.k,
                             mat.late = late.k,
                             late.excluded = !is.na(late.k),
                             env = tot.k, mmr = rep(NA, K),  
                             pm = GetPMfromME(mat.touse.k,tot.k),
                             preg = rep(NA, K), pregpm = rep(NA, K), pregmmr = rep(NA, K), 
                             issis = rep(0, K), births = rep(NA, K),
                             selogpregpm = rep(NA, length(iso.k)),
                             selogpm = rep(NA, length(iso.k)),
                             complete = complete.k, propill = propill.k, usa = usa.k,
                             include = include.k
                             ,datasource=rep("VR",length(iso.k))
                             ,datareference=rep("VR",length(iso.k))
                             ,exclude.reason=excl.k,vr.like=rep(FALSE,length(iso.k))
)


# Remove Malayasia's VR in 1997 due to data quality 
data.L[["vr"]]$include[iso.k=="MYS" & year.k==1997] <- FALSE

vradd <- read.csv(vraddfile)
vradd.tmp <- data.frame(iso = vradd$iso.code, year = vradd$year+0.5,start = vradd$year, end = vradd$year+1, 
                         data.start=vradd$year,data.end=vradd$year+1,
                         mat = vradd$maternal.deaths..n., 
                         mat.incl.late = vradd$maternal.deaths..n.,
                         mat.late = vradd$late.maternal.deaths..n.,
                         env = vradd$wra..n., mmr = rep(NA, nrow(vradd)),  
                         pm = GetPMfromME(vradd$maternal.deaths..n.,vradd$wra..n.),
                         preg = rep(NA, nrow(vradd)), pregpm = rep(NA, nrow(vradd)), pregmmr = rep(NA, nrow(vradd)), 
                         issis = rep(0, nrow(vradd)), births = rep(NA, nrow(vradd)),
                         selogpregpm = rep(NA, nrow(vradd)),
                         selogpm = rep(NA, nrow(vradd)),
                         complete = rep(NA,nrow(vradd)), propill = rep(NA,nrow(vradd)), usa = rep(NA,nrow(vradd)),
                         include = rep(TRUE,nrow(vradd))
                         ,datasource=rep("VR",nrow(vradd))
                         ,datareference=rep("VR-fromCC",nrow(vradd))
                         ,exclude.reason=rep(NA,nrow(vradd)),vr.like=rep(FALSE,nrow(vradd))
)
vradd.tmp$propill <- ifelse(is.na(vradd$ill.defined..percentage.),vradd$ill.defined..n./vradd.tmp$env,vradd$ill.defined..percentage.)
for(i in which(vradd$new.or.revised.data.point=="revised" &is.na(vradd.tmp$propill))){
  sel<-data.L[["vr"]]$iso==paste(vradd$iso.code[i]) & data.L[["vr"]]$start==vradd$year[i]
  vradd.tmp$propill[i] <- data.L[["vr"]]$propill[sel]
}

vradd.tmp$revised <- vradd$new.or.revised.data.point=="revised"
vradd.tmp <- vradd.tmp[!is.na(vradd.tmp$mat),]

vradd.tmp$late.excluded <- ifelse(is.na(vradd.tmp$mat.late), FALSE, TRUE)
vradd.tmp$mat <- ifelse(vradd.tmp$late.excluded, vradd.tmp$mat - vradd.tmp$mat.late, vradd.tmp$mat)

getc.supp <- Getc.i(iso.i = paste(vradd.tmp$iso), iso.c = meta$iso.c)
gett.supp <- Gett.i(years.i = vradd.tmp$start, year.t = meta$year.t)
for(i in 1:nrow(vradd.tmp)){
  vradd.tmp$complete[i] <- min(1,vradd.tmp$env[i]/meta$deaths.ct[getc.supp[i],gett.supp[i]])
}
vradd.tmp$usa = vradd.tmp$complete*(1-vradd.tmp$propill)
vradd.tmp$vr.like <- is.na(vradd.tmp$usa)

# "new" entries that are in current vr data base:
# paste(data.L[["vr"]]$iso,data.L[["vr"]]$start)[is.element(paste(data.L[["vr"]]$iso,data.L[["vr"]]$start),paste(vradd.tmp$iso,vradd.tmp$start)[!vradd.tmp$revised])]
# "revised entries":
# paste(data.L[["vr"]]$iso,data.L[["vr"]]$start)[is.element(paste(data.L[["vr"]]$iso,data.L[["vr"]]$start),paste(vradd.tmp$iso,vradd.tmp$start)[vradd.tmp$revised])]

# so exclude the "not revised & in vr database"
exclude <-  which(!vradd.tmp$revised & is.element(paste(vradd.tmp$iso,vradd.tmp$start), paste(data.L[["vr"]]$iso,data.L[["vr"]]$start)))
vradd.tmp[exclude,]
vradd.tmp2 <- vradd.tmp[-exclude,]
# remove the revised entry, else we can't merge with vr data
vradd.tmp2$revised <- NULL

data.L[["vr"]] <- rbind(data.L[["vr"]][
  !is.element(paste(data.L[["vr"]]$iso,data.L[["vr"]]$start),paste(vradd.tmp2$iso,vradd.tmp2$start)),],
  vradd.tmp2)

# for those countries where late are not pulled out in later years while they were in earlier years,
# late maternal deaths are used throughout
isos <- unique(data.L[["vr"]]$iso[!is.na(data.L[["vr"]]$mat.late)])
for (iso in isos){
  dat <- data.L[["vr"]][data.L[["vr"]]$iso ==iso,]
  if (sum(is.na(dat$mat.late))>0){
    if ( max(dat$year[is.na(dat$mat.late)]) > min(dat$year[!is.na(dat$mat.late)])){
      # print(iso)
      data.L[["vr"]][data.L[["vr"]]$iso ==iso,]$mat <- dat$mat.incl.late
      data.L[["vr"]][data.L[["vr"]]$iso ==iso,]$late.excluded <- FALSE
      data.L[["vr"]][data.L[["vr"]]$iso ==iso,]$pm <- GetPMfromME(dat$mat.incl.late,dat$env)
    }
  }
}

#--------------------------
# read specialized studies
datci <- read.csv(CIfile)

# Remove studies that start before 1985 and studies that have been assessed as "exclude"
exclude.i <- datci$start.year < 1985 | datci$include.in.regression.2014.2015..1...yes..2..no. == 2
exclude.i <- ifelse(is.na(exclude.i), FALSE, exclude.i) 

env <- ifelse(is.na(datci$deaths.all..deaths.to.women.15.49),datci$reported.deaths.all..deaths.to.women.15.49.as.official.reported,datci$deaths.all..deaths.to.women.15.49)
adj <- ifelse(!is.na(datci$adj.factor.maternal), datci$adj.factor.maternal, 
              datci$deaths.maternal.maternal.deaths/datci$reported.deaths.maternal..maternal.deaths.as.official.reported)

data.L[["inq"]]  <- data.frame(iso = datci$ISO3, year = 1/2*(datci$start.year + datci$end.year),
                               start = datci$start.year, end = datci$end.year,data.start=datci$start.year,data.end=datci$end.year,
                               issis = rep(0, length(datci$ISO3)), 
                               mat = datci$deaths.maternal.maternal.deaths,
                               mat.incl.late = rep(NA, length(datci$ISO3)),
                               mat.late = rep(NA, length(datci$ISO3)),
                               late.excluded = rep(NA, length(datci$ISO3)),
                               env = env,
                               births = datci$live.births,
                               mmr = datci$mmr..per.100.000.lb.*10^(-5), 
                               mmr.up = datci$mmr.up*10^(-5), mmr.low = datci$mmr.low*10^(-5),
                               pm = datci$PMDF..maternal.,
                               preg = datci$deaths.preg..pregn.related.deaths, 
                               pregmmr  = rep(NA,length(datci$ISO3)),  
                               pregpm = rep(NA, length(datci$ISO3)), 
                               adj = adj,
                               selogpm =rep(NA, length(datci$ISO3)) , selogpregpm = rep(NA, length(datci$ISO3)),
                               type = datci$X1.CI..2.Ramos..3.study, 
                               complete = ifelse(datci$incompleteness..yes.1..no.2.  ==1, "yes", "no"),
                               miscl = ifelse(datci$misclassification..yes.1..no.2. ==1, "yes", "no") ,
                               include = !exclude.i
                               ,datasource=datci$source.name
                               ,datareference=datci$reference
                               ,exclude.reason=ifelse(exclude.i==1,ifelse(datci$start.year < 1985,"pre1985","subnat"),NA)
                               ,vr.like=rep(FALSE,length(datci$ISO3))
)

#--------------------------
# read dhs

datd0 <- read.csv(dhsfile)

end.temp<-datd0$end.date
start.temp<-datd0$start.date

year <- 1/2*(start.temp+end.temp)

# if start date is before 1985, readjust the start date to reflect the calendar year around the reference year 
datd0$start.date <- ifelse(start.temp < 1985, floor(year), start.temp)
datd0$end.date <- ifelse(start.temp < 1985, floor(year)+1, end.temp)
datd0$data.start<-start.temp
datd0$data.end<-end.temp

# remove if ref year is before 1985
datd <- datd0
include.dhs<-datd0$start.date >= 1985
remove(datd0)

iso.d <- datd$iso.code
start.d <- datd$start.date
end.d <- datd$end.date
pmdf.d <- datd$pmdf
pmdfse.d <- datd$se.pmdf.calc
logpmdf.d <- datd$log.pmdf.calc - log(100)
logpmdfse.d <- datd$se.log.pmdf.calc
propmiss.d <- datd$preg.miss.pct

data.L[["dhs"]]  <- data.frame(iso = iso.d, year = 1/2*(start.d+ end.d), start = start.d, end = end.d,
                               data.start=datd$data.start,data.end=datd$data.end,
                               mat = rep(NA, length(iso.d)), 
                               mat.incl.late = rep(NA, length(iso.d)),
                               mat.late = rep(NA, length(iso.d)),
                               late.excluded = rep(NA, length(iso.d)),
                               # note: all NA for mmr and env
                               env = datd$total.deaths.obs, 
                               mmr = rep(NA, length(iso.d)), 
                               pregmmr = rep(NA, length(iso.d)), 
                               pm = rep(NA, length(iso.d)), 
                               preg = pmdf.d*datd$total.deaths.obs, 
                               pregpm = pmdf.d,
                               issis = rep(1, length(iso.d)), selogpregpm = logpmdfse.d, selogpm  = rep(NA, length(iso.d)),
                               births = rep(NA, length(iso.d)),
                               include  = include.dhs
                               ,datasource=datd$source.detail
                               ,datareference=rep(NA,length(iso.d))
                               ,exclude.reason=ifelse(include.dhs,NA,"pre1985"),vr.like=rep(FALSE,length(iso.d))
)

#--------------------------
# read misc
datm <- read.csv(miscfile)

# Discard entries with ref year before 1985 and for non-WHO countries
select <- ((datm$start.year+datm$end.year)/2 >= 1985 & is.element(datm$ISO3, meta$iso.c))
select <- ifelse(is.na(select), FALSE, select)
datm1 <- datm[select,]
remove(datm)

include.m <- (datm1$include.in.regression.2014.2015..1...yes..2..no.==1  
              & datm1$sub.population..1.yes !=1 
              & datm1$sub.national..1.yes !=1)

include.m <- ifelse(is.na(include.m), TRUE, include.m)

iso.m <- datm1$ISO3
start.m <- datm1$start.year
end.m <- datm1$end.year
# if start date is before 1985, readjust the start date to reflect the calendar year around the reference year 
year <- 1/2*(start.m+ end.m)
start.temp<-start.m
end.temp<-end.m
start.m <- ifelse(start.temp < 1985, floor(year), start.m)
end.m <- ifelse(start.temp < 1985, floor(year)+1, end.m)

typecode.m <- datm1$study.type # typenames <- c("sis and VR", "Surv", "Sis", "Civ/med/sis", "unknown", "civ/med/va", "SRS")
sis.m <- datm1$sisterhood..only...1.yes==1 
sis.m[is.na(sis.m)] <- FALSE
# get pm for mat and for preg
pm.m <- datm1$pm.mat.total.
pregovertot.m <- datm1$pm.preg.total.
pregmmr.m <- datm1$preg.ratio.pregn.deaths.births./(10^5)

mat.m <- datm1$mdeaths
mmr.m <- datm1$mmr.mat.births./10^5
births.m <- datm1$live.births
preg.m <- datm1$pregdeaths
tot.m <- datm1$deaths1549

misc.vr.like<-datm1$study.type==10 # study.type: 10 = VR
misc.vr.like[is.na(misc.vr.like)]<-FALSE

# note of reason for exclusion 
excl.m <- ifelse(include.m==1,NA,"mmeigcheck")
excl.m[which(datm1$sub.population..1.yes==1|datm1$sub.national..1.yes ==1)]<-"subnat"

data.L[["misc"]]  <- data.frame(iso = iso.m, year = 1/2*(start.m+ end.m), start = start.m, end = end.m,data.start=start.temp,data.end=end.temp,
                                mat = mat.m, 
                                mat.incl.late = rep(NA, length(iso.m)),
                                mat.late = rep(NA, length(iso.m)),
                                late.excluded = rep(NA, length(iso.m)),
                                env = tot.m, 
                                mmr = mmr.m, pregmmr  = pregmmr.m,
                                pm = pm.m, #rep(NA, length(iso.m)), 
                                preg = preg.m, 
                                selogpregpm = rep(NA, length(iso.m)), selogpm  = rep(NA, length(iso.m)),
                                pregpm = pregovertot.m,
                                issis = sis.m, 
                                births = births.m,
                                type  = typecode.m,
                                include = include.m
                                ,datasource=datm1$source.name
                                ,datareference=datm1$full.reference
                                ,exclude.reason=excl.m,vr.like=misc.vr.like
                                
)


#--------------------------
# make sure that observations are complete, e.g. infer preg from pregpm and total and vv

for (s in names(data.L)){
  l <- data.L[[s]]
  # mat deaths
  l$mat <- ifelse(is.na(l$mat), l$pm*l$env, l$mat)
  l$mat <- ifelse(is.na(l$mat), l$mmr*l$births, l$mat)
  # env
  l$env <- ifelse(is.na(l$env), l$mat/l$pm, l$env)
  l$env <- ifelse(is.na(l$env), l$preg/l$pregpm, l$env)
  
  l$births <- ifelse(is.na(l$births), l$mat/l$mmr, l$births)
  l$births <- ifelse(is.na(l$births), l$preg/l$pregmmr, l$births)
  
  # mat deaths
  l$preg <- ifelse(is.na(l$preg), l$pregpm*l$env, l$preg)
  l$preg <- ifelse(is.na(l$preg), l$pregmmr*l$births, l$preg)
  
  #REPEAT THIS AGAIN
  l$births <- ifelse(is.na(l$births), l$preg/l$pregmmr, l$births)
  
  # ratios
  l$pm <- ifelse(is.na(l$pm), GetPMfromME(l$mat,l$env), l$pm)
  l$pregpm <- ifelse(is.na(l$pregpm), (l$preg)/(l$env), l$pregpm)
  
  l$mmr <-  ifelse(is.na(l$mmr), l$mat/l$births, l$mmr)
  l$pregmmr <-  ifelse(is.na(l$pregmmr), l$preg/l$births, l$pregmmr)
  
  data.L[[s]] <- l
}

# checks if we missed out on something
# check if sum (!is.na) is ever 1 for a 3combi thing (it shouldn't be!)
which(unlist(lapply(data.L, function(l) (is.na(l$mat) + is.na(l$env) + is.na(l$pm))==1)))
which(unlist(lapply(data.L, function(l) (is.na(l$preg) + is.na(l$env) + is.na(l$pregpm))==1)))
which(unlist(lapply(data.L, function(l) (is.na(l$mat) + is.na(l$mmr) + is.na(l$births))==1)))
which(unlist(lapply(data.L, function(l) (is.na(l$preg) + is.na(l$pregmmr) + is.na(l$births))==1)))


# fix inclusion for VR for special cases

# 1. remove VR data for Colombia before 1998
COLlastexclusion <- 1997.5
exclude <- data.L[["vr"]]$iso=="COL" & data.L[["vr"]]$year <= COLlastexclusion
data.L[["vr"]]$include[exclude] <-  FALSE
excl.vr <- paste(data.L[["vr"]]$exclude.reason)
excl.vr[exclude]<-"vrdataissue"
data.L[["vr"]]$exclude.reason <- excl.vr


# 2.exclude inq and VR from South Africa 
exclude <- data.L[["vr"]]$iso=="ZAF"
data.L[["vr"]]$include[exclude] <-  FALSE
data.L[["vr"]]$exclude.reason[exclude]<-"HIV"


#---------------
# further process/clean data in data.L: 
# for inquiries:
# 1. impute maternal deaths for inquiries with adj factor only if mat vr deaths are available for the respective country-years
# 2. impute envelopes for inquiries with deaths only and missing envelopes based on vr envelope deaths or who envelopes
# 3. if a CIs includes a completeness assessment, then impute the WHO envelope for the MMR model

# for all studies:
# 4. impute pm from mmr (if mmr given only and if envelope cannot be assessed as in 2)
# 5. impute mmr from pm using UN and WHO numbers (because pm is the basis for the estimation, to see what corresponding MMR is)

# result: imputed.L with same set-up as data.L, but includes imputed values only

imputed.L <- list()
for (s in names(data.L)){
  l <- data.L[[s]]
  l$exclude.reason <- paste(l$exclude.reason)
  getc.i <- Getc.i(iso.i = paste(l$iso), iso.c = meta$iso.c)
  l2 <- NULL # this will be the imputed dataframe for source s
  l2$mmrbased <- l2$noPMwithdeath  <- rep(FALSE, length(getc.i)) 
  # used for maternal and pregn.related deaths (because same imputation of envelope)
  l2$env <- l2$preg <- l2$pm <- l2$pregpm <- l2$mat <- l2$mmr <- l2$pregmmr <- rep(NA, length(getc.i))
  
  if (s=="inq"){
    infoforinq <- list()
    # will contain info related to VR deaths and WHO deaths during inquiry period
    # index for inquiry is i, j=1,.., J.i[i] refers to the observation years during inquiry period
     infoforinq$deathswho.Li.j <- infoforinq$matvr.Li.j <- 
      infoforinq$envvrwithmissingcauses.Li.j <-  infoforinq$envvrnomissingcauses.Li.j <- list() 
    
    # total env deaths for each inquiry i    
    infoforinq$deathsforinquiry <- rep(NA, length(getc.i))
    
    # add info about VR completeness and ill-defined in midyear of inquiry

    infoforinq$complete <-  infoforinq$propill <-  rep(NA, length(getc.i))
    # and standard info
    infoforinq$iso <- l$iso
    infoforinq$start <- l$start
    infoforinq$end <- l$end
    infoforinq$mat <- l$mat    
    infoforinq$inqcapturescompl <- l$complete
    
    for (i in seq(1,length(getc.i))[l$include==TRUE]){
      # get WHO deaths 
      infoforinq$deathswho.Li.j[[i]] <- GetNumbersPerYear(start = l$start[i], end = l$end[i], 
                                                          tosum.t = meta$deaths.ct[getc.i[i],], calyear.t = meta$year.t)
      
      # get total VR envelope deaths for inquiry (if available) 
      # do NOT use inclusion criterion for the envelope deaths!!
      select <- data.L[["vr"]]$iso==paste(meta$iso.c[getc.i[i]]) 
      infoforinq$envvrwithmissingcauses.Li.j[[i]] <- GetNumbersPerYear(start = l$start[i], end = l$end[i],
                                                                       #LA20150116
                                                                       tosum.t = data.L[["vr"]]$env[select], 
                                                                       calyear.t = floor(data.L[["vr"]]$year[select]))
      # note: all NA if there is a year missing! 
      
      # get VR mat deaths (if available) 
      selectmatdeaths <- data.L[["vr"]]$iso==paste(meta$iso.c[getc.i[i]]) &  data.L[["vr"]]$include==TRUE
      infoforinq$matvr.Li.j[[i]] <- GetNumbersPerYear(start = l$start[i], end = l$end[i], 
                                                      tosum.t = data.L[["vr"]]$mat[selectmatdeaths], 
                                                      calyear.t = floor(data.L[["vr"]]$year[selectmatdeaths]))
      
      # just use midpoint for completeness and propill assessment
      selectcountryyear <- select & data.L[["vr"]]$year == (0.5+floor(mean(l$start[i],l$end[i])))
      if (sum(selectcountryyear)==1){
        infoforinq$complete[i] <-  data.L[["vr"]]$complete[selectcountryyear]
        infoforinq$propill[i] <- data.L[["vr"]]$propill[selectcountryyear]
      }
      
      # for mmr model, for any CIs that measure completeness,
      # overwrite a reported CI envelope by the WHO envelope, to make sure the mat estimates match CI estimates.
      # type is defined in read_data4.R: 1 refers to CI
      if (l$complete[i]=="yes" & l$type[i]==1){
        l2$env[i]<- sum(infoforinq$deathswho.Li.j[[i]]) # used in MMR model
        l2$pm[i] <- l$mat[i]/l2$env[i]
      }
      
      # impute envelope if missing
      if (is.na(l$env[i])){
        if (l$complete[i]=="yes"){
          infoforinq$deathsforinquiry[i] <- l2$env[i]<- sum(infoforinq$deathswho.Li.j[[i]])
        } else {
          infoforinq$deathsforinquiry[i] <-  sum(infoforinq$envvrwithmissingcauses.Li.j[[i]])
          # for MMR model:
          # we prefer to use the VR envelope but in some instances, WHO may be smaller. 
          # to avoid ending up with less mat deaths than reported in the inquiry,
          # we the min of WHO and VR 
          l2$env[i] <- min(sum(infoforinq$envvrwithmissingcauses.Li.j[[i]]),sum(infoforinq$deathswho.Li.j[[i]]))
        }
      }
      #-----------------------------
      # 1. for inquiries without pm nor mat deaths, but adj only, impute maternal deaths (if we can find the corresponding VR years to get the maternal deaths)
      if ( is.na(l$mat[i]) & !is.na(l$adj[i])){
        infoforinq$mat[i] <- l2$mat[i] <- l$adj[i]*sum(infoforinq$matvr.Li.j[[i]]) # all NAs if there are years missing
        l2$pm[i] <- l2$mat[i]/l2$env[i]
      } 
      #-----------------------------
      # 2. for inquiries with mat deaths given but not pm, find pm by imputing env deaths
      # (and keep track of what mat deaths cannot yet be used to get pm)
      if ( (!is.na(l$mat[i]) & is.na(l$pm[i]) ) |  (!is.na(l$preg[i]) & is.na(l$pregpm[i]))){ # no envelope for inquiries
        l2$pm[i] <- l$mat[i]/l2$env[i]
        l2$pregpm[i] <- l$preg[i]/l2$env[i]
        l2$noPMwithdeath[i] <- ifelse(is.na(l2$env[i]), TRUE, FALSE)
        l$exclude.reason[i]<-ifelse(is.na(l2$env[i]), "noenv", l$exclude.reason[i])
      }
    }
    
  } # end extra stuff inquiry
  
  for (i in seq(1,length(getc.i))[l$include==TRUE]){
    deathswho <- GetSums(start = l$start[i], end = l$end[i], tosum.t = meta$deaths.ct[getc.i[i],], calyear.t = meta$year.t)
    births <- GetSums(start = l$start[i], end = l$end[i], tosum.t = meta$births.ct[getc.i[i],], calyear.t = meta$year.t)
    #-----------------------------
    # use mmr to impute pm, if pm is not given and has not yet been imputed  
    if ((is.na(l$pm[i]) & is.na(l2$pm[i]) & !is.na(l$mmr[i])) ) { 
      l2$pm[i] <- l$mmr[i]*births/deathswho
      l2$mmrbased[i] <- TRUE
    }
    if (is.na(l$pregpm[i]) & is.na(l2$pregpm[i]) & !is.na(l$pregmmr[i])){ # use mmr to impute pm  
      l2$pregpm[i] <- l$pregmmr[i]*births/deathswho
      l2$mmrbased[i] <- TRUE
    }
    #-----------------------------
    # 0. obtain mmr from pm
    # 1st choice is imputed pm (to make sure we use the right one for CIs)
    # 2nd choice is data pm (if imputed one is NA)
    l2$mmr[i] <- ifelse(!is.na(l2$pm[i]), l2$pm[i]*deathswho/births, l$pm[i]*deathswho/births)
    # for preg-related, observed has first choice [actually, doesnt matter]
    l2$pregmmr[i] <- ifelse(!is.na(l$pregpm[i]), l$pregpm[i]*deathswho/births, l2$pregpm[i]*deathswho/births)
  } # end obs i in source
  names(l2) <- paste0("imputed_", names(l2))
  imputed.L[[s]] <- data.frame(l,l2)
}# end loop over sources


#--------------------------

# Next steps:
# 1. define VR model inclusion
# 2. put dataset together based on imputed.L
# 3. set final entries to NA if modelinclude is FALSE

#---------------
imputed.L[["vr"]] <- imputed.L[["vr"]][order(imputed.L[["vr"]]$iso,imputed.L[["vr"]]$year),]
# want to include all VR data included in 2014 analysis except for Mongolia due to data issues in VR
inc.14 <- read.csv(vr2014)
inc.14 <- inc.14[inc.14$iso!="MNG",] 
cy.in.inc.14<-unique(paste(inc.14$iso.code,inc.14$start.date,sep="_"))
cy.in.vr<-unique(paste(imputed.L[["vr"]]$iso,imputed.L[["vr"]]$start,sep="_"))
imputed.L[["vr"]]$include.in.2014<-is.element(cy.in.vr,cy.in.inc.14)

# 1. include VR in model when usa > cutoff AND if there are sufficient such data in a row 
imputed.L[["vr"]]$modelinclude <- imputed.L[["vr"]]$include
exclude <- ifelse(!is.na(imputed.L[["vr"]]$usa) & imputed.L[["vr"]]$usa < vrusa.cutoff, TRUE, FALSE)
imputed.L[["vr"]]$modelinclude[exclude] <-  FALSE
imputed.L[["vr"]]$exclude.reason[exclude]<-ifelse(is.na(imputed.L[["vr"]]$exclude.reason[exclude]),"lowusa",imputed.L[["vr"]]$exclude.reason[exclude])

exclude <- NULL

datavrtemp<- imputed.L[["vr"]]
for (iso in unique(datavrtemp$iso[datavrtemp$modelinclude])){
  years <- datavrtemp$year[datavrtemp$iso==iso & datavrtemp$include &datavrtemp$usa>vrusa.neighbourhood.cutoff]
  if(length(years)<3){
    exclude<-c(exclude,which(datavrtemp$iso==iso & datavrtemp$include &datavrtemp$usa>vrusa.neighbourhood.cutoff))
  }else{
    gap<-c(0,diff(years)) #difference between observation years
    group.new<-which(gap>2) #identify where new "string" of VR observation begin 
    if(length(group.new)>0){
      group<-rep(1,length(years)) #group is a vector created to store which string each VR observation belongs to 
      for(j in 1:length(group.new)){
        end<-ifelse(j==length(group.new),length(years),group.new[j+1]-1) #end is the position which the string of VR observation ends - either the end, or before the next string begins
        group[group.new[j]:end]<-group[group.new[j]:end]+j 
      }
      dropout<-rep(FALSE,length(years))
      tab<-table(group) #tabulate the groups 
      for(i in 1:max(group)){
        if(tab[i]<3){ #if each group contains less than 3 observations - dropout! 
          dropout[which(group==i)]<-TRUE
        }
      }
      exclude<-c(exclude,which(datavrtemp$iso==iso & datavrtemp$include &datavrtemp$usa>vrusa.neighbourhood.cutoff)[dropout])
    }
  }
}
  
  
# update the model inclusion column
imputed.L[["vr"]]$modelinclude[exclude] <-  FALSE
imputed.L[["vr"]]$exclude.reason[exclude]<-ifelse(is.na(imputed.L[["vr"]]$exclude.reason[exclude]),"lowusawindow",imputed.L[["vr"]]$exclude.reason[exclude])

treat.as.vrlike<-(imputed.L[["vr"]]$exclude.reason=="lowusa"|imputed.L[["vr"]]$exclude.reason=="lowusawindow")& imputed.L[["vr"]]$include.in.2014

imputed.L[["vr"]]$modelinclude[treat.as.vrlike]<-TRUE
imputed.L[["vr"]]$exclude.reason[treat.as.vrlike]<-NA
imputed.L[["vr"]]$vr.like[treat.as.vrlike]<-TRUE

#---------------------------
# 2. combine data into an array (to write to csv and use for further modeling)
# order:
# if both mat as well as preg outcomes are given (e.g. for inq or misc study), we use mat
# if there is an obs and an imputed value, we use the imputed value (e.g. envelopes may be overwritten for CIs)

# coding can be improved :)
datall <- data.frame(iso = NULL, name = NULL, year =NULL, start = NULL, end = NULL,data.start=NULL,data.end=NULL,
                     include = NULL,vr.like=NULL, 
                     type = NULL, definition = NULL, 
                     obs_mat.incl.late = NULL, obs_late.mat = NULL,late.excluded = NULL,
                     obs_mat = NULL, obs_pm = NULL,obs_env = NULL, obs_mmr = NULL,
                     obs_selogpm = NULL,
                     final_pm = NULL, final_mmr = NULL, final_env = NULL,exclude.reason=NULL,datasource=NULL,datareference=NULL)
# order of preference for using info
cats <- c("imputed_pm", "pm", "imputed_pregpm", "pregpm")
# with corresponding mmr outcomes
mcats <- c("imputed_mmr", "mmr", "imputed_pregmmr", "pregmmr")

#lapply(imputed.L, function(l) names(l))
#s <- names(imputed.L)[1]
for (s in names(imputed.L)){
  temp <- imputed.L[[s]][,c("iso", "year", "start", "end","data.start","data.end", "include","vr.like","exclude.reason","datasource","datareference","late.excluded")]
  temp$type  <- s
  
  ## late maternal deaths related 
  temp$obs_mat.incl.late <- imputed.L[[s]]$mat.incl.late
  temp$obs_late.mat <- imputed.L[[s]]$mat.late
  
  # envelopes
  temp$obs_env <- imputed.L[[s]]$env
  temp$final_env <- imputed.L[[s]]$imputed_env
  temp$final_env[is.na(temp$final_env)] <- imputed.L[[s]]$env[is.na(temp$final_env)]
  
  # determine which def will be used
  res <- imputed.L[[s]][, cats]
  cat.i <- ifelse(!is.na(res[,1]), 1,
                  ifelse(!is.na(res[,2]), 2,
                         ifelse(!is.na(res[,3]), 3, 
                                ifelse(!is.na(res[,4]), 4,NA))))
  
  temp$definition <- ifelse(!is.na(res[,1])|!is.na(res[,2])|!is.na(imputed.L[[s]][,"mat"])|!is.na(imputed.L[[s]][,"imputed_mmr"])|!is.na(imputed.L[[s]][,"mmr"]), "mat","pregn")
  
  temp$obs_mat <- ifelse(temp$definition=="pregn", imputed.L[[s]]$preg, imputed.L[[s]]$mat)
  temp$obs_pm <- ifelse(temp$definition=="pregn", imputed.L[[s]]$pregpm,imputed.L[[s]]$pm)
  temp$obs_mmr <- ifelse(temp$definition=="pregn", imputed.L[[s]]$pregmmr,imputed.L[[s]]$mmr)
  
  
  temp$final_pm <- temp$final_mmr <- rep(NA, length(cat.i))
  for (cat in unique(cat.i[!is.na(cat.i)])){
    temp$final_pm[which(cat.i==cat)] <- imputed.L[[s]][which(cat.i==cat), cats[cat]]
  }

  resm <- imputed.L[[s]][, mcats]
  mcat.i <- ifelse(!is.na(resm[,1]), 1,
                   ifelse(!is.na(resm[,2]), 2,
                          ifelse(!is.na(resm[,3]), 3, 
                                 ifelse(!is.na(resm[,4]), 4,NA))))
  
  for (mcat in unique(mcat.i[!is.na(mcat.i)])){
    temp$final_mmr[which(mcat.i==mcat)] <- imputed.L[[s]][which(mcat.i==mcat), mcats[mcat]]
    
  }
  #--------------------------#  
  # add observed SEs
  temp$obs_selogpm <- rep(NA, length(cat.i))
  # if observed pm or pregpm are used, check if there is an observed SE:
  for (cat in c(2,4)[is.element(c(2,4), unique(cat.i[!is.na(cat.i)]))]){
    temp$obs_selogpm[which(cat.i==cat)] <- imputed.L[[s]][which(cat.i==cat), c("selogpm", "selogpregpm")[cat/2]]
  }
  datall <- rbind(datall, temp)
}

# add the vr modelinclusion criteria
datall$modelinclude <- datall$include 
datall$modelinclude[datall$type=="vr"] <- imputed.L[["vr"]]$modelinclude

# exclude entries from the model with no final_pm
datall$modelinclude[is.na(datall$final_pm)] <- FALSE

# and vr usability
datall$usability_vr <- rep(NA, length(datall[,1]))
datall$usability_vr[datall$type=="vr"] <- imputed.L[["vr"]]$usa

datall$ill_vr <- rep(NA, length(datall[,1]))
datall$compl_vr <- rep(NA, length(datall[,1]))
datall$ill_vr[datall$type=="vr"] <- imputed.L[["vr"]]$propill
datall$compl_vr[datall$type=="vr"] <- imputed.L[["vr"]]$complete

datall$exclude.reason[which(datall$modelinclude==FALSE&is.na(datall$exclude.reason))]<-"noinfo"

###impute envelop for MMR based VR-like data so that the SEs can be calculated properly 
datall$final_env[datall$iso=="MNG"&datall$type=="misc"] <- (datall$obs_mat/datall$final_pm)[datall$iso=="MNG"&datall$type=="misc"]

save(datall, file = paste0("data/interm/datall.rda"))
print("Object datall written to data/interm/datall.rda")
return(NULL)


}

# The End!