
#---------------------
# Get data object to use in BMat
# Leontine Alkema and Sanqian Zhang, 2015
#---------------------

GetJagsData <- function(
  runname = "test",
  # model settings
  referenceyear = 1990,
  # default settings for time series parameters:
  sqrtgamma0max = 0.025,  
  max.sigma.lambda = 2, 
  
  # for sampling and stochastic errors
  imputeSElogPM = 0.25, 
  ## standard error on log of VR multiplier 
  stderrordelta = 0.25,
  ## warning: validationinfo is not yet fully tested in this version of the code!!
  validationinfo = list(RunValidation = FALSE, ValidateByTime = FALSE, cutoffyear = 2008, setseed = 0123)
  
){
  
  output.dir <- MakeDirs(runname)
  load(paste0(output.dir, "meta.rda"))
  load(paste0(output.dir, "datall.rda"))
  
  ## warning: validationinfo is not yet fully tested in this version of the code!!
  if (validationinfo$RunValidation){
    if(validationinfo$ValidateByTime==TRUE){
        datall$validate.exclude<-datall$year >= validationinfo$cutoffyear
    } else {
      set.seed(validationinfo$setseed) 
      datall$validate.exclude<-rbinom(n=nrow(datall),1,prob=0.20)
    }
  } else {
    datall$validate.exclude <- rep(FALSE, dim(datall)[1])
  }

  #-------------------------------------
  jagsdata <- list()
  # save (check if saved later as well)
  jagsdata$referenceyear <- referenceyear
  jagsdata$imputeSElogPM <- imputeSElogPM
  jagsdata$stderrordelta <- stderrordelta
  
  #-------------------------------------
  # 1. model par and baselist
  
  ## Time series parameters
  jagsdata$sqrtgamma0max <- sqrtgamma0max
  jagsdata$max.sigma.lambda <- max.sigma.lambda
  
  ## standard error on log of VR multiplier 
  jagsdata$stderrordelta <- stderrordelta
  
  ##aids parameters
  jagsdata$caids = meta$caids
  jagsdata$uaids = meta$uaids
  jagsdata$kaids = meta$kaids
  
  jagsdata$year.ref<-which(meta$year.t==referenceyear)
  
  X.cth <- meta$X.cth
  B.ct <- meta$births.ct
  isssa.c <- ifelse(meta$mdg.c =="Sub-Saharan Africa",1,0)

  baselist <- list(
    X.cth = X.cth, 
    isssa.c=isssa.c,
    a.ct  = meta$a.ct,
    E.ct = meta$deaths.ct, B.ct = B.ct, 
    R = meta$R, getr.c = meta$getr.c, nyears = meta$nyears, C = meta$C,
    pi.constant = pi,
    input0.ct = matrix(0, meta$C, meta$nyears),# included for restriction R <1
    v.ct = meta$v2015.ct,
    Raids.ct = meta$v2015.ct*meta$uaids,
    muaids.ct = meta$v2015.ct*meta$uaids/B.ct*meta$a.ct*meta$deaths.ct
  ) 

  #-------------------------------------
  # for VR data, excl vr-like
  isj.d<-datall$type=="vr" & datall$modelinclude & datall$vr.like==FALSE
  datavr <- datall[isj.d,]
  
  type2.j<-rep(NA,dim(datavr)[1])
  for(i in 1:dim(datavr)[1]){
    type2.j[i] <- datavr$usability_vr[i]<=0.8 
  }
  
  getj.k2<-which(type2.j)
  K2 <- length(getj.k2)
  getj.k1 <- which(type2.j==FALSE)
  K1 <- length(getj.k1)
  
  # multiplier.upperbound<-1.5+(3-1.5)*(0.8-datavr$usability_vr)/(0.2)
  delta.upperbound.j<-datavr$multiplier+(3-datavr$multiplier)*(0.8-datavr$usability_vr)/(0.2)
  
  datvrmultiplier <- list(
    getj.k2 = getj.k2,
    K2 = K2,
    getj.k1 = getj.k1,
    K1 = K1,
    delta.upperbound.j = delta.upperbound.j
  )
  
  adjpm.j <- datavr$final_pm*datavr$multiplier
  selog.j <- GetSElogPMVRinclMultUnc(pm = datavr$final_pm, mult = datavr$multiplier, env = datavr$final_env)
  bigse.j <- ifelse(datavr$usability_vr > 0.8  & selog.j > 0.5, TRUE, FALSE)
  selogpmfinal.j <- ifelse(bigse.j, 0.5, selog.j) 
  
  datvr <- list(
    logpm.j = log(datavr$final_pm),
    tau.j = 1/(selogpmfinal.j^2),
    isj.d = isj.d, ##save it for later postprocessing
    J=dim(datavr)[1], 
    getc.j = Getc.i(iso.i = paste(datavr$iso), iso.c = meta$iso.c), 
    gett.j = Gett.i(years.i = floor(datavr$year), year.t = meta$year.t), 
    mat.j  = (datavr$final_pm*datavr$final_env),
    env.j  = (datavr$final_env),
    input0forRstar.ct = matrix(0, meta$C, meta$nyears),
    input0forVR.ct = matrix(0, meta$C, meta$nyears),
    pstar.j = datavr$multiplier
  ) 
  
  #---------------
  # specialized studies with reported envelope, index is jinq
  isjinq.d<-datall$type=="inq"&!is.na(datall$final_pm)&!is.na(datall$final_env) & datall$vr.like==FALSE &datall$modelinclude
  datainq<-datall[isjinq.d,]
  
  J <- dim(datainq)[1]
  getc.j = Getc.i(iso.i = paste(datainq$iso), iso.c = meta$iso.c)
  gett.j = Gett.i(years.i = floor(datainq$year), year.t = meta$year.t) 
  # get the reference period
  start.j <- datainq$start
  end.j <- datainq$end
  gettstart.j <- Gett.i(years.i = floor(start.j), year.t = meta$year.t)
  gettend.j <- Gett.i(years.i = ceiling(end.j-1), year.t = meta$year.t)
  X.j <- gettend.j - gettstart.j +1
  partialtime.xj <- partialwhoenv.xj <- matrix(NA, max(X.j),J)
  for (j in 1:J){
    partialtime.xj[1:X.j[j],j] <- GetPartialTime(start = start.j[j], end = end.j[j], X = X.j[j])
    partialwhoenv.xj[1:X.j[j],j] <- partialtime.xj[1:X.j[j],j]*meta$deaths.ct[getc.j[j], gettstart.j[j]:gettend.j[j]] 
  }
  selogpm.jinq <- datainq$stoch_selogpm
  selogpmfinal.jinq <- ifelse(selogpm.jinq > 0.5, 0.5, selogpm.jinq)
  
  datinq <-  list(
    isjinq.d = isjinq.d,
    tau.jinq  = 1/selogpmfinal.jinq^2,
    logpm.jinq = log(datainq$final_pm),
    Jinq = J, 
    getc.jinq = getc.j, gett.jinq = gett.j, 
    gettstart.jinq = gettstart.j,
    gettend.jinq = gettend.j,
    env.jinq = (datainq$final_env),
    X.jinq = X.j, 
    partialtime.xjinq  = partialtime.xj,
    partialwhoenv.xjinq  = partialwhoenv.xj
  )
  
  #------------------------------------------------------------------------
  # VR-like, specialized study w/o envelope and other studies, index is jnew
  isjnew.d<- isj.d==FALSE & isjinq.d==FALSE & !is.na(datall$final_pm) & datall$modelinclude
  datanonvr <- datall[isjnew.d, ]
  J <- dim(datanonvr)[1]
  getc.j = Getc.i(iso.i = paste(datanonvr$iso), iso.c = meta$iso.c)
  gett.j = Gett.i(years.i = floor(datanonvr$year), year.t = meta$year.t) 
  start.j <- datanonvr$start
  end.j <- datanonvr$end
  gettstart.j <- Gett.i(years.i = floor(start.j), year.t = meta$year.t)
  gettend.j <- Gett.i(years.i = ceiling(end.j-1), year.t = meta$year.t)
  X.j <- gettend.j - gettstart.j +1
  partialtime.xj <- partialwhoenv.xj <- matrix(NA, max(X.j),J)
  for (j in 1:J){
    partialtime.xj[1:X.j[j],j] <- GetPartialTime(start = start.j[j], end = end.j[j], X = X.j[j])
    partialwhoenv.xj[1:X.j[j],j] <- partialtime.xj[1:X.j[j],j]*meta$deaths.ct[getc.j[j], gettstart.j[j]:gettend.j[j]] 
  }
  isvrlike.jnew  <- datanonvr$vr.like==1
  ismisc.jnew  <- datanonvr$vr.like!=1 & datanonvr$type!="inq"
  isinq.jnew <- datanonvr$type=="inq"
  isdhs.jnew <- datanonvr$type=="dhs"
  getjnew.k3 <- which(isvrlike.jnew)
  K3 <- length(getjnew.k3)
  getjnew.m <- which(ismisc.jnew)
  M <- length(getjnew.m)
  aids.jnew = rep(NA, J)
  for (j in 1:J) aids.jnew[j] <- baselist$v.ct[getc.j[j],gett.j[j]]*meta$a.ct[getc.j[j],gett.j[j]]
  selogpm.jnew <- datanonvr$modeled_selogpm
  # update datanonvr$modeled_selogpm to reflect new SEs for vr-like data points, 
  # and incorp initial estimate for vr multiplier
  initialvrmultiplier.k3 <- datanonvr$multiplier[getjnew.k3]
  initialvrmultiplier.k3 <- ifelse(is.na(initialvrmultiplier.k3), 1.5, initialvrmultiplier.k3)
  seforlogpm.k3 <- GetSElogPMVRinclMultUnc(pm = datanonvr$final_pm[getjnew.k3], 
                                           mult = initialvrmultiplier.k3, env =datanonvr$final_env[getjnew.k3])
  selogpm.jnew[getjnew.k3] <- seforlogpm.k3
  getk3.jnew <- rep(1, J) # not used when isvrlike=0
  getk3.jnew[getjnew.k3] <- seq(1, K3)
  
  datnonvr <-  list(
    isvrlike.jnew = isvrlike.jnew,
    ismisc.jnew = ismisc.jnew,
    isinq.jnew = isinq.jnew,
    isdhs.jnew = isdhs.jnew,
    isjnew.d = isjnew.d,
    tausamp.jnew  = 1/selogpm.jnew^2,
    logpm.jnew = log(datanonvr$final_pm),
    isssa.jnew = isssa.c[getc.j],
    Jnew = J, 
    getc.jnew = getc.j, gett.jnew = gett.j, 
    ispreg.jnew = ifelse(datanonvr$definition=="pregn",1,0), 
    gettstart.jnew = gettstart.j,
    gettend.jnew = gettend.j,
    X.jnew = X.j, 
    partialtime.xjnew  = partialtime.xj,
    partialwhoenv.xjnew  = partialwhoenv.xj
    ,getjnew.k3 = getjnew.k3
   ,initialvrmultiplier.k3 = initialvrmultiplier.k3, 
    getk3.jnew = getk3.jnew
    ,K3 = K3
   , aidsva.jnew = aids.jnew
  )
  
  #-------------------------------------
  if (validationinfo$RunValidation){
    getj.g = which(datavr$validate.exclude==TRUE)
    getjinq.ginq = which(datainq$validate.exclude==TRUE)
    getjnew.gnew = which(datanonvr$validate.exclude==TRUE)
  } else {
    getj.g = 1:datvr$J
    getjinq.ginq = 1:datinq$Jinq
    getjnew.gnew = 1:datnonvr$Jnew
  }
  
  datvalidation=list(
    Gtrain = datvr$J-sum(datavr$validate.exclude), 
    Ginqtrain = datinq$Jinq-sum(datainq$validate.exclude),
    Gnewtrain = datnonvr$Jnew-sum(datanonvr$validate.exclude),
    
    getj.gtrain = which(datavr$validate.exclude==FALSE),
    getjinq.ginqtrain = which(datainq$validate.exclude==FALSE),
    getjnew.gnewtrain = which(datanonvr$validate.exclude==FALSE),
    
    G= ifelse(validationinfo$RunValidation, sum(datavr$validate.exclude), datvr$J),
    Ginq= ifelse(validationinfo$RunValidation, sum(datainq$validate.exclude), datinq$Jinq),
    Gnew=ifelse(validationinfo$RunValidation, sum(datanonvr$validate.exclude),datnonvr$Jnew),
    
    getj.g = getj.g,
    getjinq.ginq = getjinq.ginq,
    getjnew.gnew = getjnew.gnew
    
  )

  #-------------------------------------
  jagsdata <- c(jagsdata, baselist, datvr, datinq,datnonvr
                 , datvrmultiplier
                ,datvalidation
  )

  # for constraints
  jagsdata$input1.ct <- matrix(1, meta$C, meta$nyears)
  jagsdata$input1again.ct <- jagsdata$input1.ct
  
  #-------------------------------------
  save(jagsdata, file = paste0(output.dir, "jagsdata.rda"))
  print(paste0("Object jagsdata written to ", output.dir, "jagsdata.rda"))
  return(NULL)
} # end function

#----------------------
# The End!
