
#---------------------
# domcmc.R
# Leontine Alkema and Sanqian Zhang, 2015
#-------------------------------

RunMCMC <- function(
  runname  = "test",
  # mcmc SETTINGS
  runsettings = "test", 
  ## choose from ("test", "quick", "long")
  ## test is only for checking if the model runs (just a few iterations); 
  # quick is for getting approximate results in an hour or so; 
  # long is for geting final results
  run.on.server = FALSE # using doMC and foreach libraries
){
  if (!is.element(runsettings, c("test", "quick", "long"))){
    print("You provided a non-standard option for the MCMC settings, a test run will be started.")
    runsettings <- "test"
  }
  if (runsettings=="long") print("Warning: you are starting a longggg run, which is going to take several hours of parallel computing")
  if (runsettings=="test") print("Warning: you are starting a test run, to be used for checking the model code only, do NOT use the results (they are not reliable)")
  if (runsettings=="quick") print("Warning: you are starting a quick run, which is going to take an hour or so. Results are approximate")
  
  output.dir <- MakeDirs(runname)
  load(paste0(output.dir, "meta.rda"))
  load(paste0(output.dir, "jagsdata.rda"))
  
  
  jags.params <- parnames <- c(
    "arma.ct",
    "phi", "theta", 
    "sqrtgamma0", "sigma.lambda","lambda.c","sigma.c",
    "logsigma.ar.ct","sigma.ar.ct", "mean.gamma", "sigma.gamma","phi.gamma",
    "mu.ct", "mustar.ct", "beta.h", "alpha.c", "alpha.r", "alpha.world", "sigma.country", "sigma.region", 
    "nonsamplingdhs.se","nonsamplingnondhs.se",
    "oneminpi.c", "logphi.preg.jnew", "logphi.jnew",
    "tau.jnew", # easy for showing CIs later 
    "tau.j",
    "p.wprior.k2")
  
  # Note: this is related to validation runs, not relevant (nor tested) in this public release code set
  SavePredPM <- FALSE
  if (SavePredPM){
    jags.params <- c(jags.params, "logpmpred.g", "logpmpred.ginq", "logpmpred.gnew")
  }
  
  inits <- function(AddARMA = TRUE,  meta){
    initslist <- list()
    if (AddARMA){
      initslist <- c(initslist, list("phi" = runif(1,0.6,0.8)))
    }
    # make sure we keep R below 1 for things to run
    initslist <- c(initslist, list(alpha.c = (runif(meta$C,-1,-0.2)+
        min(-
              # this is the est for Rtilde - alpha with min and max betas as defined above to get max outcome
              (- ifelse(c(meta$X.cth[,,1])<0, 1.1, 0.9)*0.4*c(meta$X.cth[,,1]) 
               + ifelse(c(meta$X.cth[,,2])<0,0.9, 1.1)*1.1*c(meta$X.cth[,,2]) 
               - ifelse(c(meta$X.cth[,,3])<0,1.1, 0.9)*0.8*c(meta$X.cth[,,3]))
      ))))
    return(list(initslist))
  } # end inits function
  
  # MCMC settings
  if (runsettings=="quick"){
    ChainNums <- seq(1,4)
    n.chains <- length(ChainNums)
    N.STEPS = 1
    n.iter.perstep = 1000
    n.thin = floor((n.iter.perstep*N.STEPS*n.chains)/3000)
    n.burnin = 2000
  }
  if (runsettings=="long"){
    ChainNums <- seq(1,10)
    n.chains <- length(ChainNums)
    N.STEPS = 15
    n.iter.perstep = 4000
    n.thin = floor((n.iter.perstep*N.STEPS*n.chains)/3000)
    n.burnin = 5000
  }
  if (runsettings=="test"){
    ChainNums <- c(1,2)
    n.chains <- length(ChainNums)
    N.STEPS = 1
    n.iter.perstep = 5
    n.thin = 1
    n.burnin = 2
  }
  
  rnorm(1)
  set.seed(1234)
  
  if (!run.on.server){
     mod <- jags.parallel(jagsdata,  jags.params, model.file = paste0(output.dir, "model.txt"), 
                          inits = inits(meta =meta),
                            n.chains=n.chains, n.burnin = n.burnin, n.iter= n.burnin+n.iter.perstep*N.STEPS, n.thin = n.thin)
     mcmc.array = mod$BUGSoutput$sims.array
  } else {
    # start parallel runs, save results in steps  
    library(foreach)
    library(doMC)
    registerDoMC()

    foreach(chainNum=ChainNums) %dopar% {
      set.seed(chainNum)
      temp <- rnorm(chainNum)
      mod <- jags(data = jagsdata, 
                  inits = inits(meta = meta),
                  parameters.to.save = jags.params,
                  n.chains = 1,
                  n.iter = n.iter.perstep+n.burnin, 
                  n.burnin = n.burnin, n.thin = n.thin, 
                  model.file= paste0(output.dir, "model.txt"),
                  jags.seed = 123*chainNum,
                  working.directory= getwd())
      i = 1 # index for which update
      mod.upd <- mod
      save(mod.upd, file=paste(output.dir, "/temp.JAGSobjects/jags_mod", chainNum, "update_", i, ".Rdata", sep = "")
      )
      cat(paste("MCMC results step", 1, " for chain ", chainNum, " written to folder temp.JAGSobjects"), "\n")
      #--- update MCMC ----------
      if (N.STEPS > 1){
        for (i in 2:(N.STEPS)){
          mod.upd <-update(mod.upd, parameters.to.save=jags.params, 
                           n.iter=n.iter.perstep,
                           n.thin=n.thin)
          save(mod.upd, file=paste(output.dir, "/temp.JAGSobjects/jags_mod", chainNum, "update_", i, ".Rdata", sep = ""))
          cat(paste("MCMC results step", i, " for chain ", chainNum, " written to folder temp.JAGSobjects"), "\n")
        }
      }
    } # end chains
    
    mcmc.array <- ReadJagsOutput(n.steps = N.STEPS, ChainNums = ChainNums, output.dir = output.dir)
  }
  save(mcmc.array, file = paste0(output.dir, "/mcmc.array.rda"))
  # bit unnecessary to save both objects but done here for convenience
  mcmc.sp <- wrap(mcmc.array, map=list(NA,3))
  save(mcmc.sp, file = paste0(output.dir, "/mcmc.sp.rda"))
  return(NULL)
} # end function

#-----

#---------------------
ReadJagsOutput <- function(n.steps, ChainNums = NULL, n.chains = NULL, output.dir = "output/"){
  if(is.null(ChainNums))  ChainNums<- seq(1, n.chains)
  if (is.null(n.chains)) n.chains <- length(ChainNums)
  chain =ChainNums[1]
  load(paste(output.dir, "temp.JAGSobjects/jags_mod", chain, "update_1.Rdata", sep = ""))
  n.iter.perstep <- dim(mod.upd$BUGSoutput$sims.array)[1]
  n.sim <- n.steps *n.iter.perstep
  n.par <- dim(mod.upd$BUGSoutput$sims.array)[3]
  mcmc.array <- array(NA, c(n.sim, n.chains, n.par))
  dimnames(mcmc.array) <- list(NULL, NULL, names(mod.upd$BUGSoutput$sims.array[1,1,]))
  for (chain in 1:n.chains){
    chain_saved <- ifelse(length(ChainNums)==1,1,ChainNums[chain])
    cat(paste("Reading in chain number ", chain_saved, " (step 1)", sep = ""), "\n")
    load(paste(output.dir,"temp.JAGSobjects/jags_mod", chain_saved, "update_1.Rdata", sep = ""))
    mcmc.array[1:n.iter.perstep,chain, ] = mod.upd$BUGSoutput$sims.array[,1,]
    if (n.steps>1){
      for (step in 2:n.steps){
        cat(paste("Reading in chain number ", chain_saved, " (step ", step, ")", sep = ""), "\n")
        load(paste(output.dir,"temp.JAGSobjects/jags_mod", chain_saved, "update_", step, ".Rdata", sep = ""))
        mcmc.array[((step-1)*n.iter.perstep+1):(step*n.iter.perstep),chain, ] = mod.upd$BUGSoutput$sims.array[,1,]
      }
    }
  }
  n.sample.max = 10000000
  if (n.sim > n.sample.max){
    mcmc.array <- mcmc.array[seq(1, n.sample.max, length.out = n.sample.max), , ]  
  }
  save(mcmc.array, file = paste(output.dir, "mcmc.array.rda", sep = ""))
  return(mcmc.array)
}


#----------------------------------------------------------------------------------
PlotTrace <- function(#Traceplot for one parameter
  ### Trace plot for one parameter and add loess smoother for each chain
  parname, mcmc.array,##<< needs to be 3-dimensional array!
  n.chains= NULL, n.sim= NULL, main = NULL){
  if (is.null(main)) main <- parname
  if (is.null(n.sim)) n.sim <- dim(mcmc.array)[1]
  if (is.null(n.chains)) n.chains <- dim(mcmc.array)[2]
  plot(c(mcmc.array[,1,parname]), type = "l", ylab = parname,  main = main,
       ylim = c(min(mcmc.array[,,parname]),max(mcmc.array[,,parname])))
  for (chain in 1:n.chains){
    lines(c(mcmc.array[,chain,parname]), type = "l", col = chain)
  }
  for (chain in 1:n.chains){
    curve(predict(loess(c(mcmc.array[,chain,parname])~seq(1,n.sim)),x), lty = 2, lwd = 3, add = TRUE, type = "l", col = chain)
  }
}

#--------------
CheckConverge <- function(runname = "test",
                          figdir = NULL,
                          percentiles = c(0.025, 0.05, 0.1, 0.5, 0.9, 0.95, 0.975)
){
  output.dir <- MakeDirs(runname)
  if (is.null(figdir)) figdir <- paste0(output.dir, "/fig")
  dir.create(figdir, showWarnings = FALSE)
  load(paste0(output.dir, "meta.rda"))
  load(paste0(output.dir, "jagsdata.rda"))
  load(paste0(output.dir, "mcmc.array.rda"))
  
  # save some plots
  print(paste("Save selected traceplots to ", figdir))
  pdf(paste0(figdir, "/trace_hyperpara_", runname, ".pdf"), width = 12, height = 12)
  par(mfrow  = c(2,2))
  for (h in 1:3){
    PlotTrace(parname = paste0("beta.h[", h, "]"), mcmc.array)
  }
  parnames <- c("sigma.region", "sigma.country", "alpha.world")
  par(mfrow  = c(2,2))
  for (parname in parnames){
    PlotTrace(parname, mcmc.array)
  }
  par(mfrow  = c(2,2))
  for (r in 1:jagsdata$R){
    PlotTrace(parname = paste0("alpha.r[", r, "]"), mcmc.array)
  }
  parnames <- c("phi", "theta","sqrtgamma0", "sigma.lambda")
  par(mfrow  = c(2,2))
  for (parname in parnames){
    PlotTrace(parname, mcmc.array)
  }
  dev.off()
  
  pdf(paste0(figdir, "/trace_ctparas_", runname, ".pdf"), width = 12, height = 12)
  par(mfrow  = c(3,2))
  randc <- sample(meta$C,60)
  for (c in 1:60){
    PlotTrace(parname = paste0("alpha.c[", randc[c], "]"), mcmc.array)
  }
  
  randc<-sample(meta$C,60,replace=TRUE)
  randt<-sample(meta$nyear,60,replace=TRUE)
  for (c in 1:60){
    PlotTrace(parname = paste0("arma.ct[", randc[c],",",randt[c], "]"), mcmc.array)
  }
  randc<-sample(meta$C,60,replace=TRUE)
  randt<-sample(meta$nyear,60,replace=TRUE)
  for (c in 1:60){
    PlotTrace(parname = paste0("mu.ct[", randc[c],",",randt[c], "]"), mcmc.array)
  }
  dev.off()
  
  
  #-------------
  print(paste("Save Rhats to  ", output.dir))
  
  ## compute R hat and post sample information ##
  post.full <- getPostInfo(mcmc.array, percentile.p = percentiles)
  write.csv(post.full, paste0(output.dir, runname, "_postinfo.csv")) 
  
  post.full["beta.h[1]", "Rhat (descending)"]
  names(post.full[, "Rhat (descending)"])[post.full[, "Rhat (descending)"]>1.05 & !is.na(post.full[, "Rhat (descending)"])]
  
  parnames <- names(post.full[, "Rhat (descending)"])[post.full[, "Rhat (descending)"]>1.05 & !is.na(post.full[, "Rhat (descending)"])]
  if (length(parnames)>0 & length(parnames)<30){
    print(paste("Save trace plots for parameters with Rhats >1.05 to  ", figdir))
    pdf(paste0(figdir, "/trace_highRhat_", runname, ".pdf"), width = 12, height = 12)
    par(mfrow  = c(2,2))
    for (parname in parnames){
      PlotTrace(parname, mcmc.array)
    }
    dev.off()
  }
  return(NULL)
} 


#----------------
# functions below are provided by Fengqing Chao
#-----------------
getPostInfo <- function(
  mcmc.array,
  percentile.p = percentiles) {
  
  ## compute R hat and median & CI for post for all para ##
  Rhat      <- Rhat(mcmc.array)
  Per       <- length(percentile.p)
  POST.INFO <- matrix(NA, nr = dim(mcmc.array)[3], nc = Per)
  for (i in 1:dim(mcmc.array)[3]) {
    post <- c(mcmc.array[, , i])
    POST.INFO[i, ] <- quantile(post, probs = percentile.p)
  }#end of i loop
  
  post.full <- cbind(POST.INFO, Rhat)
  
  # order the output from largest to smallest Rhat values
  order.para <- order(Rhat, decreasing = TRUE)
  post.full <- post.full[order.para, ]
  
  # column to show the perentiles of Rhat value
  Percentile.Rhat <- rep(NA, dim(post.full)[1])
  for (i in 1:dim(post.full)[1]) {
    Percentile.Rhat[i] <- mean(post.full[i, "Rhat"] <= Rhat) * 100
  }#end of i loop
  
  post.full <- cbind(post.full, Percentile.Rhat)
  dimnames(post.full)[[2]] <- c(paste(percentile.p * 100, "percentile"),
                                "Rhat (descending)",
                                "Percentile.Rhat (100%)")  
  return(post.full)
}#end of getPostInfo function

#-----------------
Rhat1 <- function(mat) {
  ## compute R hat for mcmc.array to check convergence ##
  m <- ncol(mat)
  n <- nrow(mat)
  b <- apply(mat, 2, mean)
  B <- sum((b - mean(mat))^2) * n / (m - 1)
  w <- apply(mat, 2, var)
  W <- mean(w)
  
  s2hat <- (n - 1) / n * W + B / n
  Vhat  <- s2hat + B / m / n 
  covWB <- n / m * (cov(w, b^2) - 2 * mean(b) * cov(w, b))
  varV  <- (n - 1)^2 / n^2 * var(w) / m +
    (m + 1)^2 / m^2 / n^2 * 2 * B^2 / (m - 1) +
    2 * (m - 1) * (n - 1) / m / n^2 * covWB
  
  df <- 2 * Vhat^2 / varV
  R  <- sqrt((df + 3) * Vhat / (df + 1) / W)
  
  return(R)
  
}#end of Rhat1 function

#-----------------
Rhat <- function(arr) {
  dm <- dim(arr)
  
  if (length(dm) == 2) return(Rhat1(arr))
  if (dm[2] == 1) return(NULL)
  if (dm[3] == 1) return(Rhat1(arr[, , 1]))
  
  return(apply(arr, 3, Rhat1))
  
}#end of Rhat function

#-----------------
# The End!
