PrepForSiblingsDHS<-function(SurveyID,InputDir,OutputDir) {
#  
# Prepares DHS individual recode and male recode (where applicable) files for processing
# of adult mortality estimates based on sibling survival histories.
# Author: Sara Hertog
# Updated by: Alison Gemmill
# Last revised: 2 July 2015
#
# Args:
#   SurveyID: character object corresponding to the column Survey.code in DHSMaster.csv 
#   InputDir: specifies the file path containing the DHS Stata files extracted from the MeasureDHS website
#   OutputDir: specifies the file path to which the output Rdata file should be written
#
# Requires:
#   foreign package for reading Stata files
#   DHSMaster.csv file in InputDir that contains information about the survey, its modules and dataset names
#   DHS individual recode file for female respondents with sibling history module

# Returns:
#   a dataframe named y with one record per sibling per 5-years of potential exposure
#   mmidx=0 identifies the respondent
#   rsex=0 identifies respondent as male=0 or female=1
#   sex.imp=TRUE indicates that siblings sex was imputed via random binomial
#   PregRelDth contains the code from mm9 variable that indicates whether female death was pregnancy-related
#   Parity is the number of live born children to the female sibling who died, not including any birth assoc with preg-related death (MM14)
#
# Example:
#   PrepForSiblingsDHS(SurveyID="cm43",
#                      InputDir="V:\\MortalitySection\\Data\\DHS\\",
#                      OutputDir="V:\\MortalitySection\\Adult Mortality\\Sibling histories\\\\")
#


  
# load foreign package to enable r to read Stata files
library(foreign) 

## Read DHS Master List to get info associated with survey indicated by SurveyID
dhs.list<-read.csv(paste(InputDir,"DHSMaster.csv",sep=""),header=TRUE,sep=",",
                     stringsAsFactors = FALSE, na.strings="")
dosurvey<-subset(dhs.list,Survey.code==SurveyID) 

######################################################################################################################
####   BEGIN LOADING DATA FILES FOR SURVEYS THAT FOLLOW THE DHS STANDARD FOR FORMAT AND VARIABLE NAMES           #####
######################################################################################################################

# Load the individual recode data file
ir.data <- read.dta(paste(InputDir,toupper(dosurvey$Individual.Recode),"FL.DTA",sep=""),
                    convert.factors=F,convert.underscore=T)

# Keep only the variables needed for sibling survival analysis
    # v001: cluster number
    # v002: household number
    # v003: respondents line number
    # v005: sample weight
    # v008: cmc date of interview
    # v011: respondent date of birth
    # v015: result of interview (1=completed)
    # v016: day of interview
    # mm1: sex of sibling
    # mm2: sibling survival status (0=dead,1=alive,8=don't know, NA=missing)
    # mm3: sibling current age (if alive; 98=don't know, NA=missing)
    # mm4: sibling date of birth (this is the century-month code calculated by DHS)
    # mm6: years ago sibling died (98=don't know, NA=missing)
    # mm7: sibling age at death (98=don't know, NA=missing)
    # mm8: sibling date of death (if dead; this is the century month code calculated by DHS)
    # mm9: pregnancy status at death (various coding schema - depends on survey)
    # mm14: number of siblings children
    # mmidx: sibling number in sibship
      
    ir.data <- ir.data[,c(grep("v001|v002|v003|v005|v008|v011|v015|v016|mm1[[:punct:]]|mm2[[:punct:]]|mm3[[:punct:]]|mm4[[:punct:]]|mm6[[:punct:]]|mm7[[:punct:]]|mm8[[:punct:]]|mm9[[:punct:]]|mm14[[:punct:]]|mmidx", names(ir.data),ignore.case=TRUE))]



# Create a unique inentifier variable for the sibling group called "mother"
  ir.data$mother <- 1:nrow(ir.data)

# Change variable names to standards used in Trevor's original sibling history code
  names(ir.data) <- sub('MM', 'mm', names(ir.data)) 
  names(ir.data) <- sub('mmIDX', 'mmidx', names(ir.data)) 
  names(ir.data) <- sub('V', 'v', names(ir.data))

######################################################################################################################
####     END LOADING DATA FILES FOR SURVEYS THAT FOLLOW THE DHS STANDARD FOR FORMAT AND VARIABLE NAMES           #####
######################################################################################################################

######################################################################################################################
####               BEGIN PROCESSING FEMALE RESPONDENTS' REPORTS OF THEIR SIBLINGS' SURVIVAL                      #####
######################################################################################################################

# Change sibling sex to 0=males 1=females, from 1=males 2=females
  ir.data[,grep("mm1[[:punct:]]", names(ir.data))] <- ir.data[,grep("mm1[[:punct:]]", names(ir.data))]-1

# Most DHS flat ir files have variables for up to 20 siblings, named with the indices 1 through 20.
# Here we add a 0 index which refers to the respondent.  Typically, the survival of respondents is not included
# when calculating adult mortality from sibling histories.  But certain adjustment procedures (e.g., Gakidou-King)
# call for including the exposure of respondents when estimating sibling survival so the necessary variables 
# are created here. 
  ir.data$sex.0 <- 1 # respondent is female (males are 0)
  ir.data$survival.0 <- 1 # responent is alive at survey
  ir.data$month_d_o_b.0 <- ir.data$v011 # respondent dob
  ir.data$month_d_o_d.0 <- NA # respondent date of death is na
  ir.data$age_curr.0<-floor((ir.data$v008-ir.data$v011)/12) # respondent age at survey
  ir.data$age_dth.0<-NA # respondent age at death is NA
  ir.data$yrsagodied.0<-NA # years ago respondent died is NA
  ir.data$mmidx.0 <- 0 # respondent is indexed 0 in sibship
  ir.data$PregRelDth.0 <- NA # respondent pregnancy status at death is NA
  ir.data$Parity.0 <- NA # respondent parity at death is NA

# Change variable names to be more meaningful and intiuitive to the analyst
  names(ir.data)[grep("mm4", names(ir.data))] <- sub('mm4', 'month_d_o_b', names(ir.data)[grep("mm4", names(ir.data))]) 
  names(ir.data)[grep("mm8", names(ir.data))] <- sub('mm8', 'month_d_o_d', names(ir.data)[grep("mm8", names(ir.data))]) 
  names(ir.data)[grep("mm1[[:punct:]]", names(ir.data))] <- sub('mm1', 'sex', names(ir.data)[grep("mm1[[:punct:]]", names(ir.data))])
  names(ir.data)[grep("mm2", names(ir.data))] <- sub('mm2', 'survival', names(ir.data)[grep("mm2", names(ir.data))])
  names(ir.data)[grep("mm3", names(ir.data))] <- sub('mm3', 'age_curr', names(ir.data)[grep("mm3", names(ir.data))])
  names(ir.data)[grep("mm6", names(ir.data))] <- sub('mm6', 'yrsagodied', names(ir.data)[grep("mm6", names(ir.data))])
  names(ir.data)[grep("mm7", names(ir.data))] <- sub('mm7', 'age_dth', names(ir.data)[grep("mm7", names(ir.data))])
  names(ir.data)[grep("mm9[[:punct:]]", names(ir.data))] <- sub('mm9', 'PregRelDth', names(ir.data)[grep("mm9[[:punct:]]", names(ir.data))])
  names(ir.data)[grep("mm14", names(ir.data))] <- sub('mm14', 'Parity', names(ir.data)[grep("mm14", names(ir.data))])

######################################################################################################################
####               END PROCESSING FEMALE RESPONDENTS' REPORTS OF THEIR SIBLINGS' SURVIVAL                        #####
######################################################################################################################



######################################################################################################################
####                 BEGIN RESHAPING THE DATA TO BE ONE RECORD PER SIBLING PER EXPOSURE PERIOD                   #####
######################################################################################################################

# The ir.data file now includes one record per respondent, with info for the respondent and up to 20 siblings
# reported on a single line.  We want to reshape the data so that it is one record per sibling 
# (includeing a record for respondent at mmidx=0)
  rsib <- reshape(ir.data, direction="long", idvar="mother", 
                 varying=list(c(grep("mmidx", names(ir.data))), 
                              c(grep("sex", names(ir.data))), 
                              c(grep("survival", names(ir.data))), 
                              c(grep("d_o_b", names(ir.data))), 
                              c(grep("d_o_d", names(ir.data))),
                              c(grep("age_curr", names(ir.data))),
                              c(grep("age_dth", names(ir.data))),
                              c(grep("yrsagodied", names(ir.data))),
                              c(grep("PregRelDth", names(ir.data))), 
                              c(grep("Parity", names(ir.data)))), 
                 v.names=c("mmidx", "sex","survival", "month_d_o_b", "month_d_o_d","age_curr","age_dth","yrsagodied","PregRelDth","Parity"))

# remove lines for nonexistents siblings
  rsib <- rsib[!is.na(rsib$mmidx),] 

# add variable for the sex of respondent who reported the sibling
  rs<-subset(rsib,mmidx==0,select=c("mother","sex"))
  rs$rsex<-rs$sex
  rs$sex<-NULL
  rsib <- merge(rsib,rs, by="mother", all.x=TRUE)

# DHS standard procedure is to impute a value for missing information on siblings'sex from
# binomial random variate generation with 1 trial and 0.5 probability of success per trial
  set.seed(100)
  rsib$sex[rsib$sex !=0 & rsib$sex !=1]<-NA
  rsib$sex.imp<-is.na(rsib$sex)
  rsib$sex[is.na(rsib$sex)] <-rbinom(nrow(rsib[is.na(rsib$sex),]),1,0.5)

# assign a random day of the month for dob
  rsib$journ <- (runif(nrow(rsib), min =0 , max = 30))/31
  rsib$d_o_b <- rsib$month_d_o_b + rsib$journ
# set dod=0 equal to missing
  rsib$month_d_o_d[rsib$month_d_o_d == 0] <- NA
# assign random day of the month for dod
  rsib$d_o_d <- rsib$month_d_o_d + (runif(nrow(rsib), min =0 , max = 30))/31

# estimate exposure for siblings who died within one month of birth
  rsib$d_o_d[which(rsib$month_d_o_d == rsib$month_d_o_b)] = rsib$d_o_b[which(rsib$month_d_o_d == rsib$month_d_o_b)] + runif(nrow(rsib[which(rsib$month_d_o_d == rsib$month_d_o_b),]), min =rsib$journ[which(rsib$month_d_o_d == rsib$month_d_o_b)], max = 30)/31
# add day of month interview occurred (v016) to month of interview (v008)
  rsib$v008 <- rsib$v008 + (rsib$v016)/31
# if interview date was before date of death then set date of death equal to interview date
  rsib$d_o_d[which(rsib$d_o_d > rsib$v008)] <- rsib$v008[which(rsib$d_o_d > rsib$v008)]
# if dob is greater than 50 years before interview date and record is for respondent then set dob equal to exactly 50 years before interview date
  rsib$d_o_b[which((rsib$d_o_b < (rsib$v008 - 50*12)) & rsib$mmidx == 0)] <- rsib$v008[which((rsib$d_o_b < (rsib$v008 - 50*12)) & rsib$mmidx == 0)] - 50*12
# if dob is less than 15 years before interview date and record is for respondent then set dob equal to exactly 15 years before the interview date
  rsib$d_o_b[which((rsib$d_o_b > (rsib$v008 - 15*12)) & rsib$mmidx == 0)] <- rsib$v008[which((rsib$d_o_b > (rsib$v008 - 15*12)) & rsib$mmidx == 0)] - 15*12
# drop interim vars from dataframe
rsib <- rsib[,-c(grep("v016|journ|month|time", names(rsib)))]
# assign a count variable equal to 1 to use in tabulation later on
  rsib$count <- 1
# calculate sizes of sibships for possible adjustment procedures later on (per Bruno Masquelier)
  rsib$rang <- ave(rsib$d_o_b, rsib$mother, FUN = rank)
  rsib$id <- rsib$mother*100 + rsib$rang
  rsib$size <- ave(rsib$count, rsib$mother, FUN = sum)


# Censor exposures of siblings
# age first is age at birth == 0 
  rsib$agefirst <- 0
# age last is age at survey date
  rsib$agelast <- trunc((rsib$v008 - rsib$d_o_b)/12)
# if dead then set agelast to age at death
  rsib$agelast[which(!is.na(rsib$d_o_d))] <- trunc(((rsib$d_o_d[which(!is.na(rsib$d_o_d))])-rsib$d_o_b[which(!is.na(rsib$d_o_d))])/12)

# Classify ages at beginning and end of exposure into 5-year age groups
  rsib$age5first[!is.na(rsib$agefirst) & rsib$agefirst < 1] <- 1
  rsib$age5first[!is.na(rsib$agefirst) & rsib$agefirst > 0] <- 2
  rsib$age5first[!is.na(rsib$agefirst) & rsib$agefirst > 4] <- trunc(rsib$agefirst[!is.na(rsib$agefirst) & rsib$agefirst > 4]/5) + 2
  
  rsib$age5last[!is.na(rsib$agelast) & rsib$agelast < 1] <- 1
  rsib$age5last[!is.na(rsib$agelast) & rsib$agelast > 0] <- 2
  rsib$age5last[!is.na(rsib$agelast) & rsib$agelast > 4] <- trunc(rsib$agelast[!is.na(rsib$agelast) & rsib$agelast > 4]/5) + 2


# Loop to get a person-period file 
  rsib <- rsib[ order(rsib$mother, rsib$rang) ,]
# duplic is the number of 5 year age groups the sib contributes exposures between birth and survey date
  rsib$duplic <- 0
  rsib$duplic <- rsib$age5last - rsib$age5first +1 
  x <- rsib[!is.na(rsib$agelast),]
# creates one record per sibling per 5 year age group of exposure in dataframe y
  y  <- (x[rep(1:nrow(x), times = x$duplic),] )
  myseq <- function(x){unlist(lapply(x,function(i){1:i}))}
# count indexes the number of 5 year age groups per sibling
  y$count <- myseq(x$duplic)
  rm(x)
# assign the sib count record to the 5-year age group
  y$age5 <- y$age5first + y$count - 1
# We can specify labels by 
  y$age <- factor(y$age5, levels = c(1:22), labels = c("0-1", "1-4", "5-9", "10-14", "15-19", "20-24", "25-29", "30-34", "35-39", "40-44", "45-49", "50-54", "55-59", "60-64", "65-69", "70-74", "75-79", "80-84", "85-89", "90-94", "95-99", "+100")) 

# remove duplic column
  y <- y[,c(-(ncol(y) - 2))]

######################################################################################################################
####                   END RESHAPING THE DATA TO BE ONE RECORD PER SIBLING PER EXPOSURE PERIOD                   #####
######################################################################################################################


# save the output as an Rdata file in the location specified by the user
save(y,file=paste(OutputDir,SurveyID,".sib.Rda",sep=""))
}
