SiblingSurvival<-function(SurveyID, recall.begin, recall.end, stderror="jackknife",InputDir,SibDir,OutputDir) {
  
# Generates estimates adult mortality rates and maternal mortality rates based on sibling survival in DHS.
# Author: Sara Hertog
  # Updated by: Alison Gemmill
  # Last revised: 2 July 2015
#
# Args:
#   SurveyID: character object corresponding to the column Survey.Code in DHSMaster.csv
#   include.MaleR: logical object indicating whether reports of sibling survival provided by
#                  male respondents (when available) should be included. Default is TRUE.
#   recall.begin: integer object indicating the maximum number of years before the survey date
#                 to begin the recall period.  To match DHS estimates 0-6 years before the
#                 survey set recall.begin=6.  For 7-13 years before the survey set recall.begin=13.
#   recall.end: integer object indicating the minimum number of years before the survey date to
#               end the recall period.  To match DHS estimates 0-6 years before the survey set
#               recall.end=0.  For 7-13 years before the survey set recall.end=7.
#   stderror: a text string that specifies which method to use to calculate standard errors
#             options so far includes "jackknife" as the default, but we should also add "bootstrap"
#   InputDir: specifies the file path containing DHSMaster file
#   SibDir: specifies the file path containing the output from "PrepForSiblingsDHS.R" function
#   OutputDir: a text string pointing to the directory in which the output should be stored
#   OutputLabel: a text string that will be attached to the name of the output csv file. It should
#                describe qualities of the parameters selected
#
# Requires:
#   Output of PrepForSiblingsDHS.r function
#
# Returns:
#   A data frame with male and female adult mortality rates by 5-year age group, q5s and 35q15s,
#   maternal mortality rates, pmdf (proportion maternal of deaths to females), log(pmdf), and the
#   percentage distribution of the female household population by age for use in calculating the pmdf for 
#   the full age range from 15 to 49.  Jackknife standard errors are calculated by removing clusters from the
#   data one at a time and recalculating the estimates, then calculating the variance of the estimates observed
#   with one cluster removed.
#
# Example:
#   SiblingSurvival(SurveyID="cm43", include.MaleR=TRUE,recall.begin=6,recall.end=0,stderror="jackknife",
#                   OutputDir="V:\\MortalitySection\\Adult Mortality\\Sibling histories\\SiblingSurvivalOutput\\",
#                   OutputLabel="test")
#
# Development notes:
#   Revised on 3 February 2014 to output male mortality rates in addition to female mortality rates

# Read DHS Master List to get info associated with survey
  dhs.list<-read.csv(paste(InputDir,"DHSMaster.csv",sep=""),header=TRUE,sep=",",
                     stringsAsFactors = FALSE, na.strings="")
  dosurvey<-subset(dhs.list,Survey.code==SurveyID) 


# load the file output by the PrepforSiblings function
 #example  #load(paste("V:\\MortalitySection\\Adult Mortality\\Sibling Histories\\Prepped sib files\\",
   #          SurveyID,".sib.Rda",sep=""))

  load(paste(SibDir, SurveyID,".sib.Rda",sep=""))



################################################################################################################
####              BEGIN DEFINE THE REFERENCE PERIOD BASED ON FUNCTION PARAMETERS                            ####
################################################################################################################


# Define the reference period
RefDate<-dosurvey$RefDate

RefYr<-as.numeric(unlist((strsplit(RefDate,"/")))[3])
RefYr <- ifelse(RefYr<100, RefYr+1900, RefYr+2000)
RefMo<-as.numeric(unlist((strsplit(RefDate,"/")))[1])
y$v008[which(is.na(y$v008))]<-((RefYr-1900)*12)+RefMo # if no interview date, then use survey ref date
y$beginmonth <- (y$v008 - (recall.begin+1)*12) #formerly currentmonth in Trevor's code
y$endmonth <- (y$v008-recall.end*12)

# these are lower and upper bounds for 5 year age groups indicated by age5
y$startage5[y$age5 == 1] <- 0 
y$endage5[y$age5 == 1] <- 1  
y$startage5[y$age5 == 2] <- 1
y$endage5[y$age5 == 2] <- 5
y$startage5[y$age5 > 2] <- 5*(y$age5[y$age5 > 2] - 2)
y$endage5[y$age5 > 2] <- 5*(y$age5[y$age5 > 2] - 2) + 5

# endperiod is the cmc that the sib ages out of the 5 year age group
y$endperiod <- y$d_o_b + y$endage5*12
# startperiod is the cmc that the sib ages into the 5 year age group
y$startperiod <- y$d_o_b + y$startage5*12

y$deces <- ifelse(!is.na(y$d_o_d) & y$d_o_d>y$startperiod & y$d_o_d>y$beginmonth &
  y$d_o_d<=y$endperiod & y$d_o_d<=y$endmonth,1,0) # did death occur in the period of exposure?
y$startexp <- pmax(y$startperiod,y$beginmonth)
y$endexp<-NA
y$endexp[which(y$deces==0)] <- pmin(y$endperiod[which(y$deces==0)],y$endmonth[which(y$deces==0)])
y$endexp[which(y$deces==1)] <- y$d_o_d[which(y$deces==1)]
y$exposure <- (y$endexp-y$startexp)/12
y$exposure[y$startperiod < y$beginmonth & y$endperiod < y$beginmonth] <- 0
y$exposure[y$startperiod > y$endmonth & y$endperiod > y$endmonth] <- 0

################################################################################################################
####                END DEFINE THE REFERENCE PERIOD BASED ON FUNCTION PARAMETERS                            ####
################################################################################################################

################################################################################################################
####              BEGIN CALCULATE ADULT FEMALE, ADULT MALE AND MATERNAL MORTALITY RATES                     ####
################################################################################################################
  
bothsexes<-y
# get mortality rates for females ages 15 to 49
y <- y[(y$sex==1) & (y$age5 > 4) & (y$age5 < 12),]
ymale <- bothsexes[(bothsexes$sex==0) & (bothsexes$age5 > 4) & (bothsexes$age5 < 12),]

# Pregnancy-related deaths
pr<-as.character(dosurvey$PregRelCode) # these have been defined by the analyst in the DHSMaster.csv file
pr1<-as.numeric(substr(pr,1,1))# currently set up to accomodate up to 5 different single-digit preg related death codes
pr2<-as.numeric(substr(pr,3,3))
pr2<-ifelse(is.na(pr2),999999,pr2)
pr3<-as.numeric(substr(pr,5,5))
pr3<-ifelse(is.na(pr3),999999,pr3)
pr4<-as.numeric(substr(pr,7,7))
pr4<-ifelse(is.na(pr4),999999,pr4)
pr5<-as.numeric(substr(pr,9,9))
pr5<-ifelse(is.na(pr5),999999,pr5)

pdth<-ifelse(y$deces==1 & (y$PregRelDth==pr1 | y$PregRelDth==pr2 | y$PregRelDth==pr3 | y$PregRelDth==pr4 | y$PregRelDth==pr5),1,0)
pdth<-ifelse(y$deces==1 & (y$PregRelDth>90 | is.na(y$PregRelDth)),NA,pdth)
y$pregdth<-pdth

# exclude index case for standard unadjusted estimate of exposure
yproxy <- y[y$mmidx > 0,]
ymale<-ymale[ymale$mmidx>0,]

# exclude if sibling sex was imputed
yproxy <- yproxy[yproxy$sex.imp==FALSE,] # female deaths and exposures
ymale <- ymale[ymale$sex.imp==FALSE,] # male deaths and exposures

# for maternal deaths and exposures, keep only those records without missing PregRelDth
ymat<-yproxy[yproxy$sex==1 & !is.na(yproxy$pregdth),]

# weight deaths and exposures according to DHS survey weights (v005)
yproxy$decesw<-yproxy$deces*yproxy$v005/1000000
yproxy$exposurew<-yproxy$exposure*yproxy$v005/1000000
ymat$pregdthw<-ymat$pregdth*ymat$v005/1000000
ymat$exposurew<-ymat$exposure*ymat$v005/1000000
ymale$decesw<-ymale$deces*ymale$v005/1000000
ymale$exposurew<-ymale$exposure*ymale$v005/1000000


# estimates for the full survey sample
  out.data<-cbind((tapply(yproxy$decesw, yproxy$age5, sum)),(tapply(yproxy$exposurew, yproxy$age5, sum)),
                  (tapply(ymat$pregdthw, ymat$age5, sum)),(tapply(ymat$exposurew, ymat$age5, sum)),
                  (tapply(ymale$decesw, ymale$age5, sum)), (tapply(ymale$exposurew, ymale$age5, sum)))
  out.data<-as.data.frame(out.data)
  colnames(out.data) <- c("deaths.female.weighted","exposure.female.weighted",
                          "matdeaths.weighted","matexposure.weighted",
                          "deaths.male.weighted","exposure.male.weighted")
  
  out.data$adultf.mort.rate<-out.data$deaths.female.weighted/out.data$exposure.female.weighted
  out.data$qxf<-(5*out.data$adultf.mort.rate)/(1+(2.5*out.data$adultf.mort.rate))
  pxf<-1-out.data$qxf
  out.data$adultm.mort.rate<-out.data$deaths.male.weighted/out.data$exposure.male.weighted
  out.data$qxm<-(5*out.data$adultm.mort.rate)/(1+(2.5*out.data$adultm.mort.rate))
  pxm<-1-out.data$qxm
  out.data$maternal.mort.rate<-out.data$matdeaths.weighted/out.data$matexposure.weighted
  out.data$pmdf<-out.data$maternal.mort.rate/out.data$adultf.mort.rate*100
  out.data$log.pmdf<-log(out.data$pmdf)
  out.data$log.pmdf[which(out.data$pmdf==0)]<-NA

  ages<-c("x15.19","x20.24","x25.29","x30.34","x35.39","x40.44","x45.49")
  out.data<-cbind(ages,out.data)

    # get hh population distribution from hh member roster file
    library(foreign) # needed for reading Stata files
    pr.data <- read.dta(paste(InputDir,toupper(dosurvey$HHMember.Recode),"FL.DTA",sep=""),
                        convert.factors=F,convert.underscore=T)
    pr.data <- subset(pr.data, hv104==2  & (hv105>=15 & hv105<=49),select=c("hv104","hv105"))
    pr.data$index <- 1
    pr.data$age5<-ifelse(pr.data$hv105<20,"x15.19",ifelse(pr.data$hv105<25,"x20.24",
                                                          ifelse(pr.data$hv105<30,"x25.29",
                                                                 ifelse(pr.data$hv105<35,"x30.34",
                                                                        ifelse(pr.data$hv105<40,"x35.39",
                                                                               ifelse(pr.data$hv105<45,"x40.44",
                                                                                      ifelse(pr.data$hv105<50,"x45.49","99")))))))
    tpr<-table(pr.data$age5)
    tpr<-as.data.frame(tpr)
    names(tpr)<-c("ages","freq")
    survey.pop.pct<-tpr$freq/(sum(tpr$freq))*100
    out.data<-cbind(out.data,survey.pop.pct)

  pmdf.15.49<-subset(out.data,ages=="x15.19")
  pmdf.15.49$ages<-"x15.49"
  pmdf.15.49$deaths.female.weighted<-sum(out.data$deaths.female.weighted)
  pmdf.15.49$exposure.female.weighted<-sum(out.data$exposure.female.weighted)
  pmdf.15.49$matdeaths.weighted<-sum(out.data$matdeaths.weighted)
  pmdf.15.49$matexposure.weighted<-sum(out.data$matexposure.weighted)
  pmdf.15.49$deaths.male.weighted<-sum(out.data$deaths.male.weighted)
  pmdf.15.49$exposure.male.weighted<-sum(out.data$exposure.male.weighted)
  pmdf.15.49$adultf.mort.rate<-pmdf.15.49$deaths.female.weighted/pmdf.15.49$exposure.female.weighted
  cpxf<-cumprod(pxf)
  pmdf.15.49$qxf<-1-(cpxf[7])
  pmdf.15.49$adultm.mort.rate<-pmdf.15.49$deaths.male.weighted/pmdf.15.49$exposure.male.weighted
  cpxm<-cumprod(pxm)
  pmdf.15.49$qxm<-1-(cpxm[7])
  pmdf.15.49$maternal.mort.rate<-pmdf.15.49$matdeaths.weighted/pmdf.15.49$matexposure.weighted
pmdf.15.49$pmdf<-(sum(out.data$maternal.mort.rate*survey.pop.pct))/(sum(out.data$adultf.mort.rate*survey.pop.pct))*100 
  pmdf.15.49$log.pmdf<-log(pmdf.15.49$pmdf)
  pmdf.15.49$survey.pop.pct<-100
  out.data<-rbind(out.data,pmdf.15.49)

################################################################################################################
####                END CALCULATE ADULT FEMALE, ADULT MALE AND MATERNAL MORTALITY RATES                     ####
################################################################################################################
  
################################################################################################################
####                             BEGIN CALCULATE JACKKNIFE STANDARD ERRORS                                  ####
################################################################################################################

# Program is set up to calulcate jackknife standard errors for PM and logPM only

# loop through excluding one cluster at a time to estimate jackknife standard errors
cluster<-table(bothsexes$v001)
cluster<-names(cluster)
jack<-NULL


survey.pop.pct.100 <- c(survey.pop.pct, 100)

for (lesscluster in c(cluster)){
pmdf<-NULL
pmdf.15.49<-NULL
use.yproxy<-subset(yproxy,v001 != lesscluster)
use.ymale<-subset(ymale,v001 != lesscluster)
use.ymat<-subset(ymat,v001 != lesscluster)
use<-cbind((tapply(use.yproxy$decesw, use.yproxy$age5, sum)),(tapply(use.yproxy$exposurew, use.yproxy$age5, sum)),
                  (tapply(use.ymat$pregdthw, use.ymat$age5, sum)),(tapply(use.ymat$exposurew, use.ymat$age5, sum)),
                  (tapply(use.ymale$decesw, use.ymale$age5, sum)), (tapply(use.ymale$exposurew, use.ymale$age5, sum)))
use<-as.data.frame(use)
colnames(use) <- c("deaths.female.weighted","exposure.female.weighted",
                   "matdeaths.weighted","matexposure.weighted",
                   "deaths.male.weighted","exposure.male.weighted")

use$adultf.mort.rate<-use$deaths.female.weighted/use$exposure.female.weighted
use$qxf<-(5*use$adultf.mort.rate)/(1+(2.5*use$adultf.mort.rate))
pxf<-1-use$qxf
use$adultm.mort.rate<-use$deaths.male.weighted/use$exposure.male.weighted
use$qxm<-(5*use$adultm.mort.rate)/(1+(2.5*use$adultm.mort.rate))
pxm<-1-use$qxm

use$maternal.mort.rate<-use$matdeaths.weighted/use$matexposure.weighted
use$pmdf<-use$maternal.mort.rate/use$adultf.mort.rate*100
use$log.pmdf<-log(use$pmdf)
use$log.pmdf[which(use$pmdf==0)]<-NA

ages<-c("x15.19","x20.24","x25.29","x30.34","x35.39","x40.44","x45.49")
use<-as.data.frame(cbind(lesscluster,ages,use), stringsAsFactors=FALSE)
pmdf.15.49<-subset(use,ages=="x15.19")
pmdf.15.49$ages<-"x15.49"
pmdf.15.49$deaths.female.weighted<-sum(use$deaths.female.weighted)
pmdf.15.49$exposure.female.weighted<-sum(use$exposure.female.weighted)
pmdf.15.49$matdeaths.weighted<-sum(use$matdeaths.weighted)
pmdf.15.49$matexposure.weighted<-sum(use$matexposure.weighted)
pmdf.15.49$deaths.male.weighted<-sum(use$deaths.male.weighted)
pmdf.15.49$exposure.male.weighted<-sum(use$exposure.male.weighted)
pmdf.15.49$adultf.mort.rate<-pmdf.15.49$deaths.female.weighted/pmdf.15.49$exposure.female.weighted
cpxf<-cumprod(pxf)
pmdf.15.49$qxf<-1-(cpxf[7])
pmdf.15.49$adultm.mort.rate<-pmdf.15.49$deaths.male.weighted/pmdf.15.49$exposure.male.weighted
cpxm<-cumprod(pxm)
pmdf.15.49$qxm<-1-(cpxm[7])
pmdf.15.49$maternal.mort.rate<-pmdf.15.49$matdeaths.weighted/pmdf.15.49$matexposure.weighted
#pmdf.15.49$pmdf<-sum((survey.pop.pct/100)*use$pmdf)
pmdf.15.49$pmdf<-(sum(use$maternal.mort.rate*survey.pop.pct))/(sum(use$adultf.mort.rate*survey.pop.pct))*100 
#pmdf.15.49$pmdf<-(sum(out.data$maternal.mort.rate*survey.pop.pct.100))/(sum(out.data$adultf.mort.rate*survey.pop.pct.100))*100 
pmdf.15.49$log.pmdf<-log(pmdf.15.49$pmdf)
use<-rbind(use,pmdf.15.49)
jack<-rbind(jack,use)
} # end loop that excludes one cluster at a time

pmdf<-subset(out.data,select=c("ages","adultf.mort.rate","qxf","adultm.mort.rate","qxm","maternal.mort.rate","pmdf","log.pmdf"))
names(pmdf)<-c("ages","amrf.theta","qxf.theta","amrm.theta","qxm.theta","mmr.theta","pmdf.theta","log.pmdf.theta")
jack<-merge(jack,pmdf,by="ages")
jack$amrf.se<-(jack$adultf.mort.rate-jack$amrf.theta)^2
jack$qxf.se<-(jack$qxf-jack$qxf.theta)^2
jack$amrm.se<-(jack$adultm.mort.rate-jack$amrm.theta)^2
jack$qxm.se<-(jack$qxm-jack$qxm.theta)^2
jack$mmr.se<-(jack$maternal.mort.rate-jack$mmr.theta)^2
jack$pmdf.se<-(jack$pmdf-jack$pmdf.theta)^2
jack$log.pmdf.se<-(jack$log.pmdf-jack$log.pmdf.theta)^2

# standard error of the adult female mortality rate
amrf.se<-tapply(jack$amrf.se, jack$ages, sum)
amrf.se<-as.data.frame(amrf.se)
amrf.stderror<-(((length(cluster)-1)/length(cluster))*(amrf.se$amrf.se))^(1/2)
out.data$adultf.mort.rate.se.jack<-amrf.stderror

# standard error of qx for females
qxf.se<-tapply(jack$qxf.se, jack$ages, sum)
qxf.se<-as.data.frame(qxf.se)
qxf.stderror<-(((length(cluster)-1)/length(cluster))*(qxf.se$qxf.se))^(1/2)
out.data$qxf.se.jack<-qxf.stderror
  
# standard error of the adult male mortality rate
amrm.se<-tapply(jack$amrm.se, jack$ages, sum)
amrm.se<-as.data.frame(amrm.se)
amrm.stderror<-(((length(cluster)-1)/length(cluster))*(amrm.se$amrm.se))^(1/2)
out.data$adultm.mort.rate.se.jack<-amrm.stderror

# standard error of qx for males
qxm.se<-tapply(jack$qxm.se, jack$ages, sum)
qxm.se<-as.data.frame(qxm.se)
qxm.stderror<-(((length(cluster)-1)/length(cluster))*(qxm.se$qxm.se))^(1/2)
out.data$qxm.se.jack<-qxm.stderror

# standard error of the maternal mortality rate
mmr.se<-tapply(jack$mmr.se, jack$ages, sum)
mmr.se<-as.data.frame(mmr.se)
mmr.stderror<-(((length(cluster)-1)/length(cluster))*(mmr.se$mmr.se))^(1/2)
out.data$maternal.mort.rate.se.jack<-mmr.stderror

# standard error of pmdf
pmdf.se<-tapply(jack$pmdf.se, jack$ages, sum)
pmdf.se<-as.data.frame(pmdf.se)
pmdf.stderror<-(((length(cluster)-1)/length(cluster))*(pmdf.se$pmdf.se))^(1/2)
out.data$pmdf.se.jack<-pmdf.stderror

# standard error of log pmdf
log.pmdf.se<-tapply(jack$log.pmdf.se, jack$ages, sum)
log.pmdf.se<-as.data.frame(log.pmdf.se)
log.pmdf.stderror<-(((length(cluster)-1)/length(cluster))*(log.pmdf.se$log.pmdf.se))^(1/2)
out.data$log.pmdf.se.jack<-log.pmdf.stderror

################################################################################################################
####                               END CALCULATE JACKKNIFE STANDARD ERRORS                                  ####
################################################################################################################

    
################################################################################################################
####              BEGIN RESHAPE THE DATA AND OUTPUT THE MORTALITY RATES TO A CSV FILE                       ####
################################################################################################################
  
library(reshape)
mout<-melt(out.data,id="ages")
rout<-reshape(mout,idvar="variable",timevar="ages", direction="wide")
names(rout)<-c("variable","x15.19","x20.24","x25.29","x30.34","x35.39","x40.44","x45.49","x15.49")

refprd<-paste(recall.end,"to",recall.begin,"years before the survey",sep=" ")
refwidth<-recall.begin-recall.end+1
survdt<-dosurvey$RefDate
refyr<-as.numeric(unlist((strsplit(RefDate,"/")))[3])
refmo<-as.numeric(unlist((strsplit(RefDate,"/")))[1])
refmo<-(refmo-1)/12
v.end.date<-(refyr+refmo)-recall.end
v.start.date<-(refyr+refmo)-(recall.begin+1)

if(dosurvey$StartYear==dosurvey$EndYear){
  survey.year<-dosurvey$StartYear
} else if (dosurvey$StartYear<dosurvey$EndYear){
  survey.year<-paste(dosurvey$StartYear,dosurvey$EndYear, sep="-")
}

source.detail<-paste(dosurvey$Type,survey.year,sep=" ")

out.adult.mort<-as.data.frame(cbind(dosurvey$iso.code,dosurvey$country.name,dosurvey$Type,
                 survey.year,
                 as.character(rout$variable),as.numeric(as.character(v.start.date)),
                               as.numeric(as.character(v.end.date)),as.numeric(rout$x15.19),
                 as.numeric(rout$x20.24),as.numeric(rout$x25.29),as.numeric(rout$x30.34),
                 as.numeric(rout$x35.39),as.numeric(rout$x40.44),as.numeric(rout$x45.49),
                               as.numeric(rout$x15.49)),
                         stringsAsFactors=FALSE)
names(out.adult.mort)<-c("iso.code","country.name","survey.name","survey.year",
                   "variable",
                   "v.start.date","v.end.date","x15-19","x20-24","x25-29",
                   "x30-34","x35-39","x40-44","x45-49","x15.49")
  
  
startdate<-dosurvey$StartDate
startyr<-as.numeric(unlist((strsplit(startdate,"/")))[3])
startmo<-as.numeric(unlist((strsplit(startdate,"/")))[1])
startmo<-(startmo-1)/12
startdate<-startyr+startmo
enddate<-dosurvey$EndDate
endyr<-as.numeric(unlist((strsplit(enddate,"/")))[3])
endmo<-as.numeric(unlist((strsplit(enddate,"/")))[1])
endmo<-(endmo-1)/12
enddate<-endyr+endmo
  out.adult.mort$v.start.date[which(out.adult.mort$variable=="survey.pop.pct")]<-startdate
  out.adult.mort$v.end.date[which(out.adult.mort$variable=="survey.pop.pct")]<-enddate

write.csv(out.adult.mort,file=paste(OutputDir,SurveyID,".output",".csv",sep=""))

return(out.adult.mort)
}


